
// Matrices

shared float4x4 ViewProj : ViewProjection < string UIWidget = "None"; >;

// Shader Constants

float4 Positions[125];
float4 Colors[125];

shared float3 SunDir = { 0.3725, 0.85, 0.3725 };
shared float3 AmbientColor = { 0.4, 0.4, 0.4 };

// Connector data structures

struct appData {
	float3 Position	: POSITION;
	float3 Normal : NORMAL;
	int4 Index : BLENDINDICES;
};

struct vertexOutput {
	float4 HPosition	: POSITION;
	float3 Color		: COLOR;
};

vertexOutput mainVS(appData IN){
	vertexOutput OUT;
	
	int index = IN.Index[0];
	
	float3 pos = Positions[index].xyz;
	float3 forward = float3(Positions[index].w, 0.0, Colors[index].w);
	float3 up = float3(0.0, 1.0, 0.0);
	float3 right = cross(forward, up);
	right = normalize(right);
	up = cross(right, forward);
	
	float4 row1 = float4(forward.x, up.x, right.x, pos.x);
	float4 row2 = float4(forward.y, up.y, right.y, pos.y);
	float4 row3 = float4(forward.z, up.z, right.z, pos.z);
	
	float4 vertPos = float4(IN.Position, 1.0);
	float4 WorldPos;
	
	WorldPos.x = dot(row1, vertPos);
	WorldPos.y = dot(row2, vertPos);
	WorldPos.z = dot(row3, vertPos);
	WorldPos.w = 1.0;
	
	/*float3 Normal;
	Normal.x = dot(row1.xyz, IN.Normal);
	Normal.y = dot(row2.xyz, IN.Normal);
	Normal.z = dot(row3.xyz, IN.Normal);*/
	
	OUT.HPosition = mul(WorldPos, ViewProj);
	//OUT.Color = (AmbientColor + (1.0 - AmbientColor) * saturate(dot(Normal, SunDir))) * abs(Colors[index].xyz);
	OUT.Color = Colors[index];
	
	return OUT;
}

float4 mainPS(vertexOutput IN) : COLOR {
	return float4(IN.Color, 1.0);
}

technique Main {
	pass p0 {
		VertexShader = compile vs_2_0 mainVS();
		ZEnable = true;
		ZWriteEnable = true;
		ZFunc = LessEqual;
		CullMode = None;
		PixelShader = compile ps_2_0 mainPS();
	}
}
