#include "APOLoc.h"
#include "Debug.h"
#include "FFT.h"
#include "HRTF.h"
#include <cmath>
#include "MemoryLog.h"

XAPO_REGISTRATION_PROPERTIES APOLoc::m_regProperties =
{
	{0x7898be97, 0x8a71, 0x49bb, {0x85, 0x4e, 0x1e, 0x28, 0xae, 0xaf, 0x32, 0xe1}},
	L"APOLoc",
	L"",
	1,
	0,
	XAPO_FLAG_FRAMERATE_MUST_MATCH
	| XAPO_FLAG_BITSPERSAMPLE_MUST_MATCH
	| XAPO_FLAG_BUFFERCOUNT_MUST_MATCH,
	1, 1, 1, 1
};

APOLoc::APOLoc():
CXAPOParametersBase(&m_regProperties, reinterpret_cast<BYTE *>(m_parameters), sizeof(*m_parameters), false),
m_workBuffer(NULL)
{
	for(unsigned i = 0; i < 3; ++i)
	{
		m_parameters[i].azim = 0.0f;
		m_parameters[i].elev = 0.0f;
	}
}

APOLoc::~APOLoc()
{
	delete[] m_workBuffer;
}

HRESULT APOLoc::LockForProcess(
	UINT32 InputLockedParameterCount, const XAPO_LOCKFORPROCESS_BUFFER_PARAMETERS* pInputLockedParameters,
	UINT32 OutputLockedParameterCount, const XAPO_LOCKFORPROCESS_BUFFER_PARAMETERS* pOutputLockedParameters)
{
	m_workBuffer = new float[FFT_NB_POINTS];

	for(unsigned i = 0; i < FFT_NB_POINTS; ++i)
	{
		m_workBuffer[i] = 0.0f;
	}

	return CXAPOBase::LockForProcess(InputLockedParameterCount, pInputLockedParameters, OutputLockedParameterCount, pOutputLockedParameters);
}

inline float * copyChannel(float *dest, const float *src, unsigned size)
{
	for(unsigned i = 0; i < size; ++i)
	{
		*dest = *src++;
		dest += 2;
	}
	return dest;
}

void APOLoc::Process(
	UINT32, const XAPO_PROCESS_BUFFER_PARAMETERS* pInputProcessParameters,
	UINT32, XAPO_PROCESS_BUFFER_PARAMETERS* pOutputProcessParameters, BOOL)
{
	APOLocParams *param = reinterpret_cast<APOLocParams *>(BeginProcess());

	switch(pInputProcessParameters[0].BufferFlags)
	{
	case XAPO_BUFFER_VALID:
		{
			float *inBuffer = static_cast<float *>(pInputProcessParameters[0].pBuffer);
			float *outBuffer = static_cast<float *>(pOutputProcessParameters[0].pBuffer);
			unsigned nbFrame = pInputProcessParameters[0].ValidFrameCount;
			std::complex<float> freqBuffer[FFT_NB_POINTS];
			float tempBuffer[FFT_NB_POINTS];

			const std::complex<float> *transfertFunc[2];
			HRTFIndex index = g_HRTF.getIndex(param->elev * 180.0f / M_PI, param->azim * 180.0f / M_PI);
			transfertFunc[0] = g_HRTF.get(index, 0);
			transfertFunc[1] = g_HRTF.get(index, 1);
			for(unsigned i = 0; i < nbFrame; i += HRTF_SAMPLE_LENGTH)
			{
				unsigned start = min(i, nbFrame - HRTF_SAMPLE_LENGTH);
				for(unsigned j = 0; j < HRTF_SAMPLE_LENGTH; ++j)
				{
					m_workBuffer[j] = m_workBuffer[j + HRTF_SAMPLE_LENGTH - (i - start)];
				}
				for(unsigned j = 0; j < HRTF_SAMPLE_LENGTH; ++j)
				{
					m_workBuffer[j + HRTF_SAMPLE_LENGTH] = inBuffer[start + j];
				}
				for(unsigned channel = 0; channel < 2; ++channel)
				{
					fastFourierTransform(m_workBuffer, freqBuffer);
					for(unsigned j = 0; j < FFT_NB_POINTS; ++j)
					{
						freqBuffer[j] *= transfertFunc[channel][j];
					}
					fastFourierTransform(freqBuffer, tempBuffer);
					copyChannel(outBuffer + 2 * start + channel, tempBuffer, HRTF_SAMPLE_LENGTH);
				}
			}
			break;
		}
	case XAPO_BUFFER_SILENT:
		break;
	}

	pOutputProcessParameters[0].ValidFrameCount = pInputProcessParameters[0].ValidFrameCount;
	pOutputProcessParameters[0].BufferFlags = pInputProcessParameters[0].BufferFlags;

	EndProcess();
}