#ifndef H_AGENT
#define H_AGENT

#include <d3dx9.h>
#include <vector>

class Junction;
class Way;
class AgentSet;
class Intersection;
class Pathfinding;
class Speaker;

class Agent
{
public:
	Agent(Speaker *speaker, Intersection &destination, Way &actualWay, AgentSet& agentSet, const Pathfinding& pathfinding);
	~Agent(void);

	void act(float timeElapsed);
	const D3DXVECTOR3& getPosition() const;
	const D3DXVECTOR3 & getColor() const;
	const Junction& getCurrentDestination() const;
	const Junction& getNextDestination() const;
	float getDistanceFromCurrentJunction() const;
	float getCurrentSpeed() const;
	const Way* getCurrentWay() const;
	Way* getNextWay(unsigned ind = 1) const;
	float getAcceleration() const;
	float getDeceleration() const;
	float getSafetyDistance() const;
	const Intersection* getNextIntersection() const;
	float getLength() const;

private:
	Agent & operator =(const Agent &);

	Intersection	*m_destination;
	Junction		*m_currentDestination;
	Way				*m_currentWay;
	const Pathfinding& m_pathfinding;
	float			 m_distanceFromCurrentJunction,
					 m_currentSpeed,
					 m_maximumSpeed,
					 m_maximumSpeedPassingJunction,
					 m_decelerationPerSecond,
					 m_accelerationPerSecond,
					 m_safetyDistance,
					 m_length;
	D3DXVECTOR3		 m_position;
	D3DXVECTOR3		 m_color;
	Speaker			*m_speaker;
	AgentSet&		 m_agentSet;

	float passToNextWay();
	const Agent* getAgentAheadOnPath() const;
	/* getNextDecDistance
	Retourne la distance parcourue par l'agent si il ralentit au maximum */
	float getNextDistanceDecelerating(float timeElapsed) const;
	float getFreeDistance() const;
	float computeAcceleration() const;
};

#endif