#ifndef H_AGENTSET
#define H_AGENTSET

#include "Buffer.h"
#include <vector>
#include <d3dx9.h>

class Agent;
class RoadNetwork;
class Renderer;
class MediaManager;
class Declaration;
class Shader;
class Graph;
class Way;
class Junction;
class Intersection;
class Pathfinding;
class SoundEngine;
class Texture;
class Control;
enum ERoadZoneType;

class AgentSet
{
public:
	AgentSet(SoundEngine *soundEngine, RoadNetwork &roadNetwork, MediaManager &media, Control *control);
	~AgentSet(void);

	void addAgent();
	void render(Renderer &render) const;
	void run(float timeElapsed);
	void reset();
	void replaceAgent(Agent* agent);
	void disposeOnRoads();
	void setShowParticles(bool visible);

	const Graph& getGraph() const;

	float getAverageVelocity()const;

protected:
	void init(int nbAgents);
	void createGeometry(std::vector<D3DXVECTOR3> &vertices, std::vector<unsigned short> &indices) const;
	void computeNormals(const std::vector<D3DXVECTOR3> &vertices, std::vector<D3DXVECTOR3> &normals, const std::vector<unsigned short> &indices) const;
	Intersection* randomDestination(ERoadZoneType type) const;
	Way* randomWay(const Intersection* dest, ERoadZoneType type) const;
	void choosePath(Way *&start, Intersection *&dest) const;
	const std::vector<Way *> & getWays(ERoadZoneType type) const;
	void computeAverageVelocity();

private:
	struct Vertex
	{
		D3DXVECTOR3 position;
		D3DXVECTOR3 normal;
		char		index[4];
	};

	struct Particle
	{
		D3DXVECTOR3 position;
		D3DXVECTOR2 texcoord;
	};


	std::vector<Agent*>					 m_agents;
	std::vector<Way *>					 m_commercialWays;
	std::vector<Way *>					 m_residentialWays;
	std::vector<Way *>					 m_allWays;
	Graph								*m_graph;
	RoadNetwork							*m_roadNetwork;
	SoundEngine							*m_soundEngine;
	// variables d'affichage
	Shader								*m_shader;
	Declaration							*m_declaration;
	VertexBuffer<Vertex>::Type			*m_vb;
	IndexBuffer<unsigned short>::Type	*m_ib;

	Shader								*m_shaderParticles;
	Declaration							*m_declParticles;
	VertexBuffer<Particle>::Type		*m_vbParticles;
	IndexBuffer<unsigned short>::Type	*m_ibParticles;
	Texture								*m_textureParticles;

	float								 m_nbAgents;
	float								 m_undefinedPathProp;
	float								 m_resToComProp;
	float								 m_highwayRatio;
	float								 m_timeCoeff;

	MediaManager						*m_media;
	Pathfinding							*m_pathfinding;
	bool								 m_disposed;
	bool								 m_showParticles;
	float								 m_averageVelocity;
};

#endif