#include "Allotment.h"
#include "MemoryLog.h"

Allotment::Allotment(ZoneType type, const std::vector<D3DXVECTOR3> &vertices):
m_type(type)
{
	unsigned size = vertices.size();
	m_edges.resize(vertices.size());
	for(unsigned i = 0; i < size; i++)
	{	
		m_edges[i].index = i;
		m_edges[i].marker = false;
		m_edges[i].v1 = vertices[i % size];
		m_edges[i].v2 = vertices[(i+1) % size];
		m_edges[i].vector = m_edges[i].v2 - m_edges[i].v1;
		m_edges[i].length = D3DXVec3Length(&m_edges[i].vector);

		D3DXVECTOR3 pV = D3DXVECTOR3(m_edges[i].vector.z, 0.0f, - m_edges[i].vector.x);
		D3DXVec3Normalize(&m_edges[i].n, &pV);
	}
	m_vertices = vertices;
	m_position = gravityCenter();

}

Allotment::Allotment(ZoneType type, const std::vector<D3DXVECTOR3> &vertices, const std::vector<bool> &markers):
m_type(type)
{
	unsigned size = vertices.size();
	m_edges.resize(vertices.size());
	for(unsigned i = 0; i < size; i++)
	{	
		m_edges[i].index = i;
		m_edges[i].marker = markers[i];
		m_edges[i].v1 = vertices[i % size];
		m_edges[i].v2 = vertices[(i+1) % size];
		m_edges[i].vector = m_edges[i].v2 - m_edges[i].v1;
		m_edges[i].length = D3DXVec3Length(&m_edges[i].vector);

		D3DXVECTOR3 pV = D3DXVECTOR3(m_edges[i].vector.z, 0.0f, - m_edges[i].vector.x);
		D3DXVec3Normalize(&m_edges[i].n, &pV);
	}
	m_vertices = vertices;
	m_position = gravityCenter();
}

Allotment::~Allotment(void)
{
}

D3DXVECTOR2 Allotment::gravityCenter()
{
	std::vector<D3DXVECTOR3> &vertices = getVertices();
	int n = vertices.size();
	D3DXVECTOR2 sumA1Ai = D3DXVECTOR2(0.0f, 0.0f);
	for(int i = 0; i < n; i++)
		sumA1Ai += D3DXVECTOR2(vertices[i].x, vertices[i].z);
	return sumA1Ai / static_cast<float>(n);
}

std::vector<Edge> & Allotment::getEdges(void)
{
	return m_edges;
}

std::vector<D3DXVECTOR3> & Allotment::getVertices(void)
{
	return m_vertices;
}

D3DXVECTOR2 Allotment::getPosition()
{
	return m_position;
}

void Allotment::addVertex(const D3DXVECTOR3 &v)
{
	m_vertices.push_back(v);
}

void Allotment::addEdge(const Edge &e)
{
	m_edges.push_back(e);	
}

bool Allotment::isIn()
{
	std::vector<Edge> edges = m_edges;
	for(unsigned i = 0; i < edges.size(); i++)
	{
		if(!edges[i].marker) return false;
	}
	return true;
}

ZoneType Allotment::getZoneType() const
{
	return m_type;
}