#include "Appli.h"
#include <exception>
#include "MediaManager.h"
#include "Renderer.h"
#include "GUI.h"
#include "CommandFrame.h"
#include "SoundEngine.h"
#include "HRTF.h"
#include "Speaker.h"
#include "MemoryLog.h"

#define WINDOW_STYLE	WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN | WS_CLIPSIBLINGS
#define WINDOW_STYLE_EX	WS_EX_OVERLAPPEDWINDOW

/******************************************************************************************************************************
Constructeur / Destructeur
******************************************************************************************************************************/
Appli::Appli(HINSTANCE hInst):
m_hInstance(hInst),
m_width(APP_WIDTH),
m_height(APP_HEIGHT),
m_d3d(NULL),
m_d3dDevice(NULL),
m_fullscreen(false),
m_pause(false),
m_run(true),
m_deviceLost(false)
{
	g_HRTF.load("./Data/Sound/HRTF/");
	/****** Window Class ******/
	// cration
	WNDCLASSEX tmp = {
        sizeof( WNDCLASSEX ), 
		CS_OWNDC | CS_VREDRAW | CS_HREDRAW,		// style de classe
		msgProc,								// le gestionnaire d'vnements
		0L,										// nombre de bytes allous aprs l'espace rserv  la window class
		0L,										// nombre de bytes allous aprs l'espace rserv  l'instance de notre fentre
        GetModuleHandle( NULL ), 
		LoadIcon(NULL, IDI_WINLOGO),				// l'icone de l'application
		LoadCursor(NULL, IDC_ARROW),			// curseur de l'application
		NULL, 
		NULL,
		CLASS_NAME, 
		NULL
	};
	m_wc = tmp;
	// enregistrement
    RegisterClassEx( &m_wc );

	/****** Fentre ******/
	RECT rect = { 0, 0, m_width, m_height };
	AdjustWindowRectEx(&rect, WINDOW_STYLE, false, WINDOW_STYLE_EX);
	// cration de la fentre
	m_hwnd = CreateWindowEx(	WINDOW_STYLE_EX,
								CLASS_NAME,				// nom de la window Class
								APP_NAME,				// nom de l'application
								WINDOW_STYLE,	// fentre avec bouton fermer, rduire ...
								CW_USEDEFAULT,			// position en x
								CW_USEDEFAULT,			// position en y
								rect.right - rect.left,	// longeur
								rect.bottom - rect.top,	// largeur
								NULL, 
								NULL, 
								m_wc.hInstance, 
								this 
							);

	/****** initialisation Direct3D ******/
	initD3D();
#ifndef NO_SOUND
	m_soundEngine = new SoundEngine();
#endif

	m_gui = new GUI(m_iomanager, *m_mediaManager, *this);
	CommandFrame *cmdFrame = new CommandFrame(m_gui);

	/****** Cration de City ******/
	m_city = new City(m_soundEngine, *m_mediaManager, cmdFrame);

	ShowWindow(m_hwnd, SW_SHOW);
}

Appli::~Appli(void)
{
	delete m_city;
	delete m_mediaManager;
#ifndef NO_SOUND
	delete m_soundEngine;
#endif
	delete m_gui;

	destructD3D();
}

/******************************************************************************************************************************
Fonctions prives
******************************************************************************************************************************/

void Appli::initD3D()
{
    // Create the D3D object, which is needed to create the D3DDevice.
    if( NULL == ( m_d3d = Direct3DCreate9( D3D_SDK_VERSION ) ) )
		throw std::bad_exception("Can't create Direct3D device");

	D3DPRESENT_PARAMETERS param = createParameters();
	unsigned vertexprocessing = D3DCREATE_HARDWARE_VERTEXPROCESSING;

	if(FAILED(m_d3d->CheckDeviceType(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL,
		param.BackBufferFormat, param.BackBufferFormat, param.Windowed)))
	{
		vertexprocessing = D3DCREATE_SOFTWARE_VERTEXPROCESSING;
	}

    // on tente de crer un device
	if( FAILED( m_d3d->CreateDevice(	D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, m_hwnd,
										vertexprocessing,
										&param, 
										&m_d3dDevice ) ) )
	{
		throw std::bad_exception("Can't create Direct3D device");
	}

	m_mediaManager = new MediaManager(*new Renderer(m_d3dDevice));
}

void Appli::destructD3D()
{
	m_d3dDevice->Release();
	m_d3d->Release();
}

void Appli::restoreD3D()
{
	int hr = m_d3dDevice->TestCooperativeLevel();
	if(D3DERR_DEVICENOTRESET == hr || D3D_OK == hr)
	{
		/* Sauvegarde du contenu de la mmoire vido */
		// CUSTOMVERTEX *data = new CUSTOMVERTEX[3];
		// CUSTOMVERTEX *temp = NULL;
		// m_VB->Lock(0, 0, reinterpret_cast<void **>(&temp), D3DLOCK_READONLY);
		// memcpy(data, temp, 3 * sizeof(CUSTOMVERTEX));
		// m_VB->Unlock();
		// m_VB->Release();
		m_mediaManager->onLostDevice();
		D3DPRESENT_PARAMETERS param = createParameters();
		// on reprcise les paramtres
		if (D3D_OK == m_d3dDevice->Reset(&param))
		{
			// le reset est effectu, on rcupre la mmoire
			/* recopie des donnes dans la mmoire vido */
			// m_d3dDevice->CreateVertexBuffer( 3 * sizeof( CUSTOMVERTEX ),
			//	0, D3DFVF_CUSTOMVERTEX,
			//	D3DPOOL_DEFAULT, &m_VB, NULL );
			// m_VB->Lock(0, 0, reinterpret_cast<void **>(&temp), D3DLOCK_READONLY);
			// memcpy(temp, data, 3 * sizeof(CUSTOMVERTEX));
			// m_VB->Unlock();
			// on signale que le device n'est plus perdu
			m_mediaManager->onResetDevice();
			m_deviceLost = false;
		}
		// on supprime la copie mmoire centrale de la mmoire vido
		// delete[] data;
	}
}

D3DPRESENT_PARAMETERS Appli::createParameters()
{
	D3DPRESENT_PARAMETERS d3dpp;
	ZeroMemory( &d3dpp, sizeof( d3dpp ) );

	// rcupration de la rsolution
	if (m_fullscreen)
	{
		// on commence par rcuprer la rsolution actuelle de l'cran
		D3DDISPLAYMODE dm;
		m_d3d->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &dm);
		// puis on l'assigne  notre appli
		d3dpp.BackBufferWidth = dm.Width;
		d3dpp.BackBufferHeight = dm.Height;
		d3dpp.BackBufferFormat = dm.Format;
		d3dpp.FullScreen_RefreshRateInHz = dm.RefreshRate;

		// TODO: trouver un meilleur endroit pour faire a
		m_width = dm.Width;
		m_height = dm.Height;
		m_gui->onSize(m_width, m_height);
	}
	else // windowed
	{
		d3dpp.BackBufferWidth = 0;
		d3dpp.BackBufferHeight = 0;
		d3dpp.BackBufferFormat = D3DFMT_UNKNOWN;
		d3dpp.FullScreen_RefreshRateInHz = 0;
	}	

	d3dpp.PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;

	d3dpp.EnableAutoDepthStencil = TRUE;
	d3dpp.AutoDepthStencilFormat = D3DFMT_D24X8;
	d3dpp.hDeviceWindow = m_hwnd;
	d3dpp.Windowed = !m_fullscreen;
	d3dpp.BackBufferCount = 1; 
	// mise en place du mulit-sample, si possible
	d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;

	if(SUCCEEDED(m_d3d->CheckDeviceMultiSampleType(
		D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, D3DFMT_D24X8,
		!m_fullscreen, D3DMULTISAMPLE_8_SAMPLES, NULL)))
	{
		d3dpp.MultiSampleType = D3DMULTISAMPLE_8_SAMPLES;
	}
	else if(SUCCEEDED(m_d3d->CheckDeviceMultiSampleType(
		D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, D3DFMT_D24X8,
		!m_fullscreen, D3DMULTISAMPLE_4_SAMPLES, NULL)))
	{
		d3dpp.MultiSampleType = D3DMULTISAMPLE_4_SAMPLES;
	}
	else if(SUCCEEDED(m_d3d->CheckDeviceMultiSampleType(
		D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, D3DFMT_D24X8,
		!m_fullscreen, D3DMULTISAMPLE_2_SAMPLES, NULL)))
	{
		d3dpp.MultiSampleType = D3DMULTISAMPLE_2_SAMPLES;
	}
	else
	{
		d3dpp.MultiSampleType = D3DMULTISAMPLE_NONE;
	}

	return d3dpp;
}

bool Appli::proceedMsg(UINT msg, WPARAM wParam, LPARAM lParam)
{ 
	switch( msg )
    {
		// destruction de la fentre
        case WM_DESTROY:
            // TODO : cleanup
			m_run = false;
            PostQuitMessage( 0 );
            return true;
		// redimensionnement de la fentre
		case WM_SIZE:
			// on regarde le type de redimensionnement
			switch (wParam)
			{
				// passage plein cran
				case SIZE_MAXIMIZED:
					// on enlve la pause, et on passe en plein cran
					m_pause = false;
					restoreD3D();
					break;
				// rduction dans la barre des tches
				case SIZE_MINIMIZED:
					// on met le logiciel en pause, et on sort du plein cran
					m_pause = true;
					break;
				// autre redimensionnement
				case SIZE_RESTORED:
					// on enlve la pause et on sort du plein cran
					m_pause = false;
					restoreD3D();
					break;
			}
			m_width = LOWORD(lParam);
			m_height = HIWORD(lParam);
			m_gui->onSize(m_width, m_height);

			return true;
		// touche appuye
		case WM_KEYDOWN:
			m_iomanager.injectKeyDown(wParam);
			return true;					
		// touche relache
		case WM_KEYUP:
			m_iomanager.injectKeyUp(wParam);
			return true;
		case WM_CHAR:
			m_iomanager.injectKeyChar(wParam);
			return true;
		// souris bouge
		case WM_MOUSEMOVE:
			m_iomanager.injectMouseMove(GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam));
			return true;
		// molette souris
		case WM_MOUSEWHEEL:
			m_iomanager.injectMouseWheel(GET_WHEEL_DELTA_WPARAM(wParam));
			return true;
		// boutons de la souris appuys
		case WM_LBUTTONDOWN:
			m_iomanager.injectMouseClick(MK_LBUTTON);
			return true;
		case WM_RBUTTONDOWN:
			m_iomanager.injectMouseClick(MK_RBUTTON);
			return true;
		case WM_MBUTTONDOWN:
			m_iomanager.injectMouseClick(MK_MBUTTON);
			return true;
		// boutons de la souris relachs:
		case WM_LBUTTONUP:
			m_iomanager.injectMouseRelease(MK_LBUTTON);
			return true;
		case WM_RBUTTONUP:
			m_iomanager.injectMouseRelease(MK_RBUTTON);
			return true;
		case WM_MBUTTONUP:
			m_iomanager.injectMouseRelease(MK_MBUTTON);
			return true;
    }
	return false;
}

LRESULT CALLBACK Appli::msgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{	
	// en cas de message CREATE (cration de la fentre), on initialise le pointeur fentre.
	if (msg == WM_CREATE)
	{
		SetWindowLongPtr (	hWnd, 
							GWLP_USERDATA, 
							reinterpret_cast<LONG_PTR> ( (reinterpret_cast<CREATESTRUCT*> (lParam))->lpCreateParams	)
						 );
	}
	
	// on rcupre un pointeur vers notre fentre.
	Appli *app = reinterpret_cast<Appli*> (GetWindowLongPtr(hWnd, GWLP_USERDATA));
	// on traite le message reu
	if(app)
	{
		app->proceedMsg(msg, wParam, lParam);
	}

    return DefWindowProc( hWnd, msg, wParam, lParam );
}

/******************************************************************************************************************************
Fonctions publiques
******************************************************************************************************************************/

void Appli::run()
{
	// boucle principale
	MSG msg = {0};
	LARGE_INTEGER lastTick, actualTick, frequency;
	float timeElapsed;

	QueryPerformanceCounter(&lastTick);
	QueryPerformanceFrequency(&frequency);
	while(m_run)
	{
		// gestion des ventuels messages clavier
		while(PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ))
		{
			TranslateMessage( &msg );
			DispatchMessage( &msg );
		}

		// on mesure le temps depuis le dernier passage
		QueryPerformanceCounter(&actualTick);
		timeElapsed = static_cast<float>(actualTick.QuadPart - lastTick.QuadPart) / frequency.QuadPart;

		if(timeElapsed > 0.2f)
		{
			timeElapsed = 0.2f;
		}
		
		// on met  jours l'ioManager mme si on est ne pause
		m_iomanager.update(timeElapsed);
		m_gui->update(timeElapsed);

		// mise  jours du contexte par les managers, si on n'est pas en pause
		if (!m_pause && !m_deviceLost)
		{
			// mise  jours du contexte
			update(timeElapsed);

			// si le device est perdu...
			if (D3DERR_DEVICELOST == render())
			{
				// on essaye de le restaurer
				m_deviceLost = true;
			}
		}
		// si le device est perdu, on tente de le restaurer
		if (m_deviceLost)
		{
			if (D3DERR_DEVICENOTRESET == m_d3dDevice->TestCooperativeLevel())
			{
				restoreD3D();
			}
			else
			{
				Sleep(200);
			}
		}

		lastTick = actualTick;
	}
}

HRESULT Appli::render()
{

	// si le device n'existe pas, on considre qu'il a t perdu
    if( NULL == m_d3dDevice )
        return D3DERR_DEVICELOST;

    m_d3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB( 128, 128, 255 ), 1.0f, 0 );

    // Dbut de la scne
    if( SUCCEEDED( m_d3dDevice->BeginScene() ) )
    {
		initializeMatrix();
		/* Code d'affichage */
		m_city->render(m_mediaManager->getRenderer());
		m_gui->render(m_mediaManager->getRenderer());
        // fin de la scne
        m_d3dDevice->EndScene();
    }

    // envoie le backbuffer  l'affichage. On retourne le rsultat de cette fonction, qui nous dira si le device est 
	// toujours disponible
    return m_d3dDevice->Present( NULL, NULL, NULL, NULL );
}

void Appli::update (float timeElapsed)
{
	// on met  jours la ville
	m_city->update(timeElapsed);

#ifndef NO_SOUND
	m_soundEngine->update(timeElapsed, m_camera);
#endif
}

void Appli::initializeMatrix()
{
	D3DXMATRIX proj, world, view, temp;
	float ratio = static_cast<float>(m_width) / static_cast<float>(m_height);
	D3DXMatrixPerspectiveFovLH( &proj, D3DX_PI / 4.0f, ratio, 0.1f, 250.0f );
	D3DXMatrixIdentity(&world);
	view = m_camera.getViewMatrix();

	m_mediaManager->getRenderer().getPool().setValue("Proj", &proj);
	m_mediaManager->getRenderer().getPool().setValue("View", &view);
	proj = view * proj;
	m_mediaManager->getRenderer().getPool().setValue("ViewProj", &proj);
	m_mediaManager->getRenderer().getPool().setValue("Width", static_cast<float>(m_width));
	m_mediaManager->getRenderer().getPool().setValue("Height", static_cast<float>(m_height));
	m_mediaManager->getRenderer().setMatrix(world);
}

Camera & Appli::getCamera()
{
	return m_camera;
}

void Appli::toggleFullscreen()
{
	m_fullscreen = !m_fullscreen;

	if(!m_fullscreen)
	{
		SetWindowLong(m_hwnd, GWL_STYLE, WINDOW_STYLE);
		SetWindowLong(m_hwnd, GWL_EXSTYLE, WINDOW_STYLE_EX);

		restoreD3D();

		SetWindowPos(m_hwnd, HWND_NOTOPMOST,
			m_windowedRect.left,
			m_windowedRect.top,
			m_windowedRect.right - m_windowedRect.left,
			m_windowedRect.bottom - m_windowedRect.top,
			SWP_SHOWWINDOW);
	}
	else
	{
		GetWindowRect(m_hwnd, &m_windowedRect);
		SetWindowLong(m_hwnd, GWL_STYLE, WS_POPUP);
		SetWindowLong(m_hwnd, GWL_EXSTYLE, 0);

		restoreD3D();
	}
}

void Appli::exit()
{
	m_run = false;
}