#ifndef H_APPLI
#define H_APPLI

#include <windows.h>
#include <windowsx.h>
#include <d3d9.h>
#include "IOManager.h"
#include "City.h"
#include "Camera.h"

#define CLASS_NAME "C_CityGen"
#define APP_NAME "CityGen"
#define APP_WIDTH 1000
#define APP_HEIGHT 750

class Landscape;
class MediaManager;
class SoundEngine;
class GUI;

class Appli
{
private:
	HINSTANCE			 m_hInstance;
	WNDCLASSEX			 m_wc;
	HWND				 m_hwnd;
	IOManager			 m_iomanager;
	int					 m_width,
						 m_height;
	RECT				 m_windowedRect;
	bool				 m_fullscreen,
						 m_pause,
						 m_run,
						 m_deviceLost;
	LPDIRECT3D9			 m_d3d; // Used to create the D3DDevice
	LPDIRECT3DDEVICE9	 m_d3dDevice; // Our rendering device
	MediaManager		*m_mediaManager;
	SoundEngine			*m_soundEngine;
	City				*m_city;
	Camera				 m_camera;	
	GUI					*m_gui;

	/* initD3D
	Initialise Direct3D. Retourne vrai si D3D a t correctement initialis, faux sinon */
	void initD3D();
	/* releaseD3D
	Suprprimes les ressources D3D. */
	void destructD3D();
	/* restoreD3DDevice
	Recr le device D3D aprs la perte de celui-ci */
	void restoreD3D();
	/* createParameters
	Retourne un D3DPRESENT_PARAMETERS valide pour la situation actuelle de l'appli */
	D3DPRESENT_PARAMETERS createParameters();
	/* msgProc
	Handler des messages de la fentre */
	static LRESULT CALLBACK msgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
	/* proceedMsg
	Traite un message reu. Retourne vrai si le message a t trait, faux sinon. */
	bool proceedMsg(UINT msg, WPARAM wParam, LPARAM lParam);

	void initializeMatrix();

public:
	Appli(HINSTANCE hInst);
	~Appli();
	/* run
	Lance la boucle principale */
	void run();
	/* render
	Effectue le rendu de la frame */
	HRESULT render();
	/* update
	Fonction de mise  jours du contexte */
	void update(float timeElapsed);

	Camera & getCamera();
	void toggleFullscreen();
	void exit();
};

#endif
