#include "Block.h"
#include "Building.h"
#include <cmath>
#include "MemoryLog.h"


Block::Block(std::vector<D3DXVECTOR3>& vertices, ZoneType type):
m_type(type)
{
	D3DXVECTOR3 lastPoint = vertices[0];
	m_vertices.push_back(vertices[0]);
	for(unsigned i = 0; i < vertices.size(); ++i)
	{
		D3DXVECTOR3 diff(vertices[i] - lastPoint);
		if(D3DXVec3LengthSq(&diff) > 0.000001f)
		{
			lastPoint = vertices[i];
			m_vertices.push_back(lastPoint);
		}
	}
}

Block::~Block(void)
{
	// on dtruit les btiments que le block contient
	for (std::vector<Building*>::iterator it = m_buildings.begin(); it != m_buildings.end(); ++it)
	{
		delete *it;
	}
	m_buildings.clear();
}

/**************************************************************************************************
					Getters et Setters
**************************************************************************************************/

const std::vector<D3DXVECTOR3>& Block::getVertices() const
{
	return m_vertices;
}
std::vector<D3DXVECTOR3>& Block::getVertices()
{
	return m_vertices;
}

const std::vector<Building*>& Block::getBuildings() const
{
	return m_buildings;
}

ZoneType Block::getZoneType() const
{
	return m_type;
}

void Block::setType(const ZoneType type)
{
	m_type = type;
}

void Block::addBuilding(Building &building)
{
	m_buildings.push_back(&building);
}

void Block::addVertex(const D3DXVECTOR3 &vertex)
{
	m_vertices.push_back(vertex);
}

bool Block::pointInside(const D3DXVECTOR2 &point) const
{
	float sum = 0.0f;
	for (unsigned i=0; i < m_vertices.size(); i++)
	{
		D3DXVECTOR2 vOP1 = D3DXVECTOR2(m_vertices[i].x, m_vertices[i].z) - point,
					vOP2 = D3DXVECTOR2(m_vertices[(i+1)%m_vertices.size()].x, m_vertices[(i+1)%m_vertices.size()].z) - point;
		D3DXVec2Normalize(&vOP1, &vOP1);
		D3DXVec2Normalize(&vOP2, &vOP2);

		float angle = D3DXVec2Dot(&vOP1, &vOP2);
		angle = (angle > 1.0f) ? 1.0f : (angle < -1.0f) ? -1.0f : angle;
		angle = std::acos(angle);
		if (D3DXVec2CCW(&vOP1, &vOP2) < 0.0f)
		{
			angle = -angle;
		}
		sum += angle;
	}

	return std::abs(sum) >= 2 * D3DX_PI;
}