#ifndef H_BLOCKSET
#define H_BLOCKSET

#include <vector>
#include "Buffer.h"
#include <d3dx9.h>

class Block;
class MediaManager;
class Renderer;
class Declaration;
class Shader;

#define NB_PATCHES_BLOCKSET 4

class BlockSet
{
public:
	BlockSet(MediaManager &media);
	~BlockSet(void);

	void addBlock(Block &b);

	void commitChange();
	void render (Renderer &renderer) const;
	void reset();

	std::vector<Block*>& getBlocks();
	const std::vector<Block *> & getBlocks() const;

protected:
	void recomputeBuffer();

private:
	struct Vertex
	{
		D3DXVECTOR3 Position;
		D3DXVECTOR3 Color;
	};

	struct Patch
	{
		VertexBuffer<Vertex>::Type			*vb;
		IndexBuffer<unsigned short>::Type	*ib;
		std::vector<Block *>				 newBlocks;
		std::vector<Block *>				 blocks;
		unsigned short						 nbPoints;

		Patch();
	};


	Shader								*m_shader;
	Declaration							*m_declaration;
	MediaManager						*m_media;
	Patch								 m_patches[NB_PATCHES_BLOCKSET*NB_PATCHES_BLOCKSET];
	std::vector<Block *>				 m_blocks;
};

#endif