#include "MemoryLog.h"

template <class T, class Type>
Buffer<T, Type>::Buffer(unsigned size, Type buffer):
m_size(size),
m_buffer(buffer)
{
}

template <class T, class Type>
Buffer<T, Type>::~Buffer()
{
	if(m_buffer)
	{
		m_buffer->Release();
	}
}

template <class T, class Type>
T * Buffer<T, Type>::lock()
{
	T *data = NULL;
	if(FAILED(m_buffer->Lock(0, 0, reinterpret_cast<void**>(&data), 0)))
	{
		return NULL;
	}
	return data;
}

template <class T, class Type>
T * Buffer<T, Type>::lock(unsigned offset)
{
	T *data = NULL;
	if(FAILED(m_buffer->Lock(offset * sizeof(T), (m_size - offset) * sizeof(T), reinterpret_cast<void**>(&data), 0)))
	{
		return NULL;
	}
	return data;
}

template <class T, class Type>
T * Buffer<T, Type>::lock(unsigned offset, unsigned size)
{
	T *data = NULL;
	if(FAILED(m_buffer->Lock(offset * sizeof(T), size * sizeof(T), reinterpret_cast<void**>(&data), 0)))
	{
		return NULL;
	}
	return data;
}

template <class T, class Type>
void Buffer<T, Type>::unlock()
{
	m_buffer->Unlock();
}

template <class T, class Type>
void Buffer<T, Type>::fill(const T *data, unsigned size)
{
	T *buf = lock(0, size);
	for(unsigned i = 0; i < size; ++i)
	{
		*buf++ = *data++;
	}
	unlock();
}

template <class T, class Type>
unsigned Buffer<T, Type>::getSize() const
{
	return m_size;
}

template <class T, class Type>
Type Buffer<T, Type>::getBuffer() const
{
	return m_buffer;
}

#undef new