#ifndef BUILDINGSET
#define BUILDINGSET

#include <vector>
#include "Buffer.h"
#include "ZoneType.h"
#include <d3dx9.h>

class Building;
class MediaManager;
class Renderer;
class Declaration;
class Shader;

#define NB_PATCHES_BUILDINGSET 4

class BuildingSet
{
public:
	BuildingSet(MediaManager &media);
	~BuildingSet(void);

	void addBuilding(Building &b, ZoneType type);

	void commitChange();
	void render (Renderer &renderer) const;
	void reset();

	void setHeight(float);

protected:
	void recomputeBuffer();

private:
	struct Vertex
	{
		D3DXVECTOR3 Position;
		D3DXVECTOR3 Normal;
	};

	struct Patch
	{
		VertexBuffer<Vertex>::Type			*vb;
		IndexBuffer<unsigned short>::Type	*ib;
		std::vector<Building *>				 newBuildings;
		std::vector<Building *>				 buildings;
		unsigned short						 nbVertices;
		unsigned short						 nbIndices;

		Patch();
	};


	Shader								*m_shader;
	Declaration							*m_declaration;
	MediaManager						*m_media;
	Patch								 m_patches[NB_ZONE_TYPE * NB_PATCHES_BUILDINGSET * NB_PATCHES_BUILDINGSET];
	std::vector<Building*>				 m_buildings;
	float								 m_height;
};
#endif