#include "Camera.h"
#include "IOManager.h"
#include <cmath>
#include "MemoryLog.h"

#define WHEEL_SPEED 0.02f
#define ROTATE_SPEED 0.002f
#define TRANSLATE_KEY_SPEED	32.0f
#define TRANSLATE_MOUSE_SPEED 0.1f

Camera::Camera( void ):
m_position( 3.075f, -76.5f, 7.8f )
{
	m_rotations[0] = -0.786f;
	m_rotations[1] = -0.72f;
	update();
}

Camera::~Camera( void )
{
}

const D3DXVECTOR3& Camera::getPosition( void ) const
{
	return m_position;
}

void Camera::setPosition( const D3DXVECTOR3& p )
{
	m_position = p;
}

void Camera::rotateX(float var)
{
	m_rotations[0] -= var;

	if(m_rotations[0] > D3DX_PI / 2.0f)
	{
		m_rotations[0] = D3DX_PI / 2.0f;
	}
	else if(m_rotations[0] < -D3DX_PI / 2.0f)
	{
		m_rotations[0] = -D3DX_PI / 2.0f;
	}
}

void Camera::rotateY(float var)
{
	m_rotations[1] -= var;
}

void Camera::translateX(float var)
{  
	m_position.x -= cos(m_rotations[1]) * var;
	m_position.z -= sin(m_rotations[1]) * var;
}

void Camera::translateZ(float var)
{
	float r = std::cos(m_rotations[0]);
	m_position.x += std::sin(m_rotations[1]) * r * var;
	m_position.y -= std::sin(m_rotations[0]) * var;
	m_position.z -= std::cos(m_rotations[1]) * r * var;	
}
	
void Camera::translateY(float var)
{
	float r = std::sin(m_rotations[0]);
	m_position.x += std::sin(m_rotations[1]) * r * var;
	m_position.y += std::cos(m_rotations[0]) * var;
	m_position.z -= std::cos(m_rotations[1]) * r * var;
}

const D3DXMATRIX& Camera::getViewMatrix( void ) const
{
	return m_viewMatrix;
}

void Camera::update()
{
	D3DXMATRIX temp, rotX, rotY, trans;
	D3DXMatrixMultiply(
		&m_viewMatrix,
		D3DXMatrixTranslation(&trans, m_position.x, m_position.y, m_position.z),
		D3DXMatrixMultiply(
		   &temp,
		   D3DXMatrixRotationY(&rotY, m_rotations[1]),
		   D3DXMatrixRotationX(&rotX, m_rotations[0])
		));
}