#ifndef H_CITY
#define H_CITY

#include <vector>
#include "Environment.h"
#include "Density.h"
#include "Road.h"
#include "BlockSet.h"
#include "BuildingSet.h"
#include "RoadNetwork.h"
#include "Geometry.h"
#include "GUIEvent.h"
#include "AgentSet.h"

class MediaManager;
class Renderer;
class Generator;
class CommandFrame;

class City
{
public:
	City(SoundEngine *soundEngine, MediaManager &media, CommandFrame *frame);
	~City(void);

	void render (Renderer &renderer) const;
	void update (float timeElapsed);

	void runAll();
	void run();
	void runStep();
	void reset();

	void setupAgents();
	void startSimulation();
	void stopSimulation();
	void showBottlenecks(bool show);

protected:
	void setRenderState();
	void setGenerators();
	void deleteGenerators();
	void createControls();
	void attachControls();

private:
	Environment					 m_environment;	
	Density						 m_density;	
	RoadNetwork					 m_roadNetwork;
	Geometry					 m_roadGeometry;
	BlockSet					 m_blocks;
	BlockSet					 m_allotments;
	BuildingSet					 m_buildings;
	std::vector<Generator *>	 m_pipeline;
	std::vector<unsigned>		 m_renderStates;
	unsigned					 m_actStep;	
	AgentSet					 m_agentSet;
	// variables d'affichage
	MediaManager				*m_mediaManager;
	CommandFrame				*m_interface;
	bool						 m_run;
	bool						 m_stopAtNextStep;
};

#endif