#include "CommandFrame.h"
#include "GUI.h"
#include "GUIButton.h"
#include "GUIListBox.h"
#include "GUILabel.h"
#include "Control.h"
#include <fstream>
#include <sstream>
#include <iomanip>
#include "MemoryLog.h"

static const char * const s_modeTitle[CFM_NB_MODES] =
{
	"Generation",
	"Simulation"
};

#define AVERAGESPEED_TEXT "Average Speed: "
#define DEFAULT_AVERAGESPEED_TEXT AVERAGESPEED_TEXT "N/A"

CommandFrame::CommandFrame(GUI *gui):
GUIFrame(gui, "CommandFrame", gui->getRoot(), s_modeTitle[0])
{
	setType(FT_DOCKLEFT);
	getSize().setRelativeSize(D3DXVECTOR2(250.0f, 0.0f), D3DXVECTOR2(0.0f, 1.0f));

	for(unsigned i = 0; i < CFM_NB_MODES; ++i)
	{
		m_panels[i] = new GUIWindow(gui, s_modeTitle[i], this);
		m_panels[i]->getSize().setRelativeSize(D3DXVECTOR2(0.0f, 0.0f), D3DXVECTOR2(1.0f, 1.0f));

		if(i != 0)
		{
			m_panels[i]->hide();
		}
	}

	// Paneau generation
	GUIButton *saveButton = new GUIButton(gui, "saveButton", m_panels[0], "Save");
	saveButton->getSize().setRelativeSize(D3DXVECTOR2(-20.0f, 32.0), D3DXVECTOR2(0.5f, 0.0f));
	saveButton->getPosition().setPosition(D3DXVECTOR2(16.0f, 16.0f), GA_TOPLEFT, GA_TOPLEFT);

	GUIButton *loadButton = new GUIButton(gui, "loadButton", m_panels[0], "Load");
	loadButton->getSize().setRelativeSize(D3DXVECTOR2(-20.0f, 32.0), D3DXVECTOR2(0.5f, 0.0f));
	loadButton->getPosition().setPosition(D3DXVECTOR2(-16.0f, 16.0f), GA_TOPRIGHT, GA_TOPRIGHT);

	m_list = new GUIListBox(gui, "stepList", m_panels[0]);
	m_list->getSize().setRelativeSize(D3DXVECTOR2(-32.0f, 12.0f), D3DXVECTOR2(1.0f, 0.0f));
	m_list->getPosition().setPosition(D3DXVECTOR2(0.0f, 56.0f), GA_TOP, GA_TOP);
	m_list->onItemSelected += new BlindDelegate<CommandFrame, int>(this, &CommandFrame::selectControl);

	m_buttons = new GUIWindow(gui, "buttonsControl", m_panels[0]);
	m_buttons->getSize().setRelativeSize(D3DXVECTOR2(-32.0f, 72.0f), D3DXVECTOR2(1.0f, 0.0f));

	GUIButton *runButton = new GUIButton(gui, "runButton", m_buttons, "Run");
	runButton->getSize().setAbsoluteSize(D3DXVECTOR2(64.0f, 32.0f));
	runButton->getPosition().setPosition(D3DXVECTOR2(0.0f, 0.0f), GA_TOPLEFT, GA_TOPLEFT);

	GUIButton *stepButton = new GUIButton(gui, "stepButton", m_buttons, "Step");
	stepButton->getSize().setAbsoluteSize(D3DXVECTOR2(64.0f, 32.0f));
	stepButton->getPosition().setPosition(D3DXVECTOR2(0.0f, 0.0f), GA_TOP, GA_TOP);

	GUIButton *runAllButton = new GUIButton(gui, "startButton", m_buttons, "All");
	runAllButton->getSize().setAbsoluteSize(D3DXVECTOR2(64.0f, 32.0f));
	runAllButton->getPosition().setPosition(D3DXVECTOR2(0.0f, 0.0f), GA_TOPRIGHT, GA_TOPRIGHT);

	GUIButton *clearButton = new GUIButton(gui, "clearButton", m_buttons, "Clear");
	clearButton->getSize().setRelativeSize(D3DXVECTOR2(0.0f, 32.0f), D3DXVECTOR2(1.0f, 0.0f));
	clearButton->getPosition().setPosition(D3DXVECTOR2(0.0f, 40.0f), GA_TOP, GA_TOP);

	saveButton->onRelease += new BlindDelegate<CommandFrame, void>(this, &CommandFrame::saveParameters);
	loadButton->onRelease += new BlindDelegate<CommandFrame, void>(this, &CommandFrame::loadParameters);
	stepButton->onClick += new Delegate<GUIEvent<void>, void>(&onStep, &GUIEvent<void>::operator ());
	runButton->onClick += new Delegate<GUIEvent<void>, void>(&onRun, &GUIEvent<void>::operator ());
	runAllButton->onClick += new Delegate<GUIEvent<void>, void>(&onRunAll, &GUIEvent<void>::operator ());
	clearButton->onClick += new Delegate<GUIEvent<void>, void>(&onClear, &GUIEvent<void>::operator ());

	//Paneau Simulation
	saveButton = new GUIButton(gui, "saveButton", m_panels[1], "Save");
	saveButton->getSize().setRelativeSize(D3DXVECTOR2(-20.0f, 32.0), D3DXVECTOR2(0.5f, 0.0f));
	saveButton->getPosition().setPosition(D3DXVECTOR2(16.0f, 16.0f), GA_TOPLEFT, GA_TOPLEFT);

	loadButton = new GUIButton(gui, "loadButton", m_panels[1], "Load");
	loadButton->getSize().setRelativeSize(D3DXVECTOR2(-20.0f, 32.0), D3DXVECTOR2(0.5f, 0.0f));
	loadButton->getPosition().setPosition(D3DXVECTOR2(-16.0f, 16.0f), GA_TOPRIGHT, GA_TOPRIGHT);

	GUIButton *resetButton = new GUIButton(gui, "resetButton", m_panels[1], "Reset City");
	resetButton->getSize().setRelativeSize(D3DXVECTOR2(-32.0f, 32.0f), D3DXVECTOR2(1.0f, 0.0f));
	resetButton->getPosition().setPosition(D3DXVECTOR2(0.0f, 56.0f), GA_TOP, GA_TOP);

	GUIButton *setupButton = new GUIButton(gui, "setupButton", m_panels[1], "Setup Agents");
	setupButton->getSize().setRelativeSize(D3DXVECTOR2(-32.0f, 32.0f), D3DXVECTOR2(1.0f, 0.0f));
	setupButton->getPosition().setPosition(D3DXVECTOR2(0.0f, 96.0f), GA_TOP, GA_TOP);

	GUIButton *startButton = new GUIButton(gui, "startButton", m_panels[1], "Start");
	startButton->getSize().setRelativeSize(D3DXVECTOR2(-20.0f, 32.0), D3DXVECTOR2(0.5f, 0.0f));
	startButton->getPosition().setPosition(D3DXVECTOR2(16.0f, 136.0f), GA_TOPLEFT, GA_TOPLEFT);

	GUIButton *stopButton = new GUIButton(gui, "stopButton", m_panels[1], "Stop");
	stopButton->getSize().setRelativeSize(D3DXVECTOR2(-20.0f, 32.0), D3DXVECTOR2(0.5f, 0.0f));
	stopButton->getPosition().setPosition(D3DXVECTOR2(-16.0f, 136.0f), GA_TOPRIGHT, GA_TOPRIGHT);

	m_simulationControl = new Control(gui, "simulation");
	m_panels[1]->addChild(m_simulationControl);
	m_simulationControl->getSize().setRelativeSizeX(-32.0f, 1.0f);
	m_simulationControl->getPosition().setPosition(D3DXVECTOR2(0.0f, 176.0f), GA_TOP, GA_TOP);

	GUIButton *bottleneckButton = new GUIButton(gui, "bottleneckButton", m_panels[1], "View Bottlenecks");
	bottleneckButton->getSize().setRelativeSize(D3DXVECTOR2(-32.0f, 32.0f), D3DXVECTOR2(1.0f, 0.0f));
	bottleneckButton->getPosition().setPosition(D3DXVECTOR2(0.0f, -16.0f), GA_BOTTOM, GA_BOTTOM);
	bottleneckButton->setTogglable(true);

	m_averageSpeedLabel = new GUILabel(gui, "averageSpeed", m_panels[1], DEFAULT_AVERAGESPEED_TEXT);
	m_averageSpeedLabel->getPosition().setPosition(D3DXVECTOR2(20.0f, -56.0f), GA_BOTTOMLEFT, GA_BOTTOMLEFT);

	saveButton->onRelease += new BlindDelegate<CommandFrame, void>(this, &CommandFrame::saveParameters);
	loadButton->onRelease += new BlindDelegate<CommandFrame, void>(this, &CommandFrame::loadParameters);
	resetButton->onRelease += new Delegate<GUIEvent<void>, void>(&onClear, &GUIEvent<void>::operator ());
	setupButton->onClick += new Delegate<GUIEvent<void>, void>(&onSetupAgents, &GUIEvent<void>::operator ());
	startButton->onClick += new Delegate<GUIEvent<void>, void>(&onStartSimulation, &GUIEvent<void>::operator ());
	stopButton->onClick += new Delegate<GUIEvent<void>, void>(&onStopSimulation, &GUIEvent<void>::operator ());
	bottleneckButton->onPress += new Delegate<GUIEvent<bool>, bool>(&onShowBottlenecks, &GUIEvent<bool>::operator ());

	refreshControlsPosition();
}

void CommandFrame::setCurrentStage(int index)
{
	for(int i = 0; i < index; ++i)
	{
		m_list->setActive(i, false);
	}

	for(int i = index; i < static_cast<int>(m_list->getNbElements()); ++i)
	{
		m_list->setActive(i, true);
	}

	if(index < static_cast<int>(m_list->getNbElements()))
	{
		m_list->select(index);
	}
}

void CommandFrame::setMode(ECommandFrameMode mode)
{
	for(int i = 0; i < CFM_NB_MODES; ++i)
	{
		if(i == mode)
		{
			m_panels[i]->show();
			setTitle(m_panels[i]->getName());
		}
		else
		{
			m_panels[i]->hide();
		}
	}
}

void CommandFrame::addControl(Control *control)
{
	m_list->addElement(control->getName());
	m_panels[0]->addChild(control);
	control->hide();
	control->getSize().setRelativeSizeX(-32.0f, 1.0f);
	m_controls.push_back(control);

	refreshControlsPosition();
}

void CommandFrame::refreshControlsPosition()
{
	m_list->autoSize();
	D3DXVECTOR2 pos = D3DXVECTOR2(0.0f, 64.0f + m_list->getSize().getAbsoluteSize().y);
	m_buttons->getPosition().setPosition(pos, GA_TOP, GA_TOP);
	pos.y += m_buttons->getSize().getAbsoluteSize().y + 8.0f;
	
	for(unsigned i = 0; i < m_controls.size(); ++i)
	{
		m_controls[i]->getPosition().setPosition(pos, GA_TOP, GA_TOP);
	}
}

void CommandFrame::selectControl(int index)
{
	for(int i = 0; i < static_cast<int>(m_controls.size()); ++i)
	{
		if(i == index)
		{
			m_controls[i]->show();
		}
		else
		{
			m_controls[i]->hide();
		}
	}
}

void CommandFrame::saveParameters()
{
	char currentDirectory[512];
	GetCurrentDirectory(512, currentDirectory);

	char buffer[512] = "";
	OPENFILENAME filename = { 0 };
	filename.lStructSize = sizeof(filename);
	filename.lpstrFilter = "City param\0*.cty\0";
	filename.lpstrFile = buffer;
	filename.nMaxFile = 512;
	filename.lpstrInitialDir = "..\\..\\..\\Data\\Save";
	filename.lpstrDefExt = "cty";

	if(GetSaveFileName(&filename))
	{
		std::ofstream file(filename.lpstrFile);
		for(unsigned i = 0; i < m_controls.size(); ++i)
		{
			m_controls[i]->saveParameters(file);
			file << "\n";
		}
		file.close();
	}

	SetCurrentDirectory(currentDirectory);
}

void CommandFrame::loadParameters()
{
	char currentDirectory[512];
	GetCurrentDirectory(512, currentDirectory);

	char buffer[512] = "";
	OPENFILENAME filename = { 0 };
	filename.lStructSize = sizeof(filename);
	filename.lpstrFilter = "City param\0*.cty\0";
	filename.lpstrFile = buffer;
	filename.nMaxFile = 512;
	filename.lpstrInitialDir = "..\\..\\..\\Data\\Save";
	filename.lpstrDefExt = "cty";
	filename.Flags = OFN_FILEMUSTEXIST;

	if(GetOpenFileName(&filename))
	{
		std::ifstream file(filename.lpstrFile);
		for(unsigned i = 0; i < m_controls.size(); ++i)
		{
			m_controls[i]->loadParameters(file);
		}
		file.close();
	}

	SetCurrentDirectory(currentDirectory);
	onClear(this);
}

Control * CommandFrame::getControl(unsigned index) const
{
	return m_controls[index];
}

Control * CommandFrame::getSimulationControl() const
{
	return m_simulationControl;
}

void CommandFrame::setAverageSpeed(float averageSpeed)
{
	if(averageSpeed < 0.0f)
	{
		m_averageSpeedLabel->setText(DEFAULT_AVERAGESPEED_TEXT);
	}
	else
	{
		std::ostringstream stream;
		stream << AVERAGESPEED_TEXT << std::setprecision(2) << std::setiosflags(std::ios_base::fixed) << averageSpeed << " u/s";
		m_averageSpeedLabel->setText(stream.str());
	}
}