#ifndef H_CONTROL
#define H_CONTROL

#include "GUIWindow.h"
#include "HashMap.h"
#include <fstream>

class GUILabel;
class GUITrackbar;

class Control: public GUIWindow
{
public:
	Control(GUI *gui, const std::string &name);

	void addParameter(const std::string &name, const std::string &label);
	void addParameter(const std::string &name, const std::string &label, float minValue, float maxValue);
	void addParameter(const std::string &name, const std::string &label, float minValue, float maxValue, float defaultValue);
	void addParameter(const std::string &name, const std::string &label, float minValue, float maxValue, float defaultValue, float step);

	void attachValue(const std::string &name, float &value);

	void saveParameters(std::ofstream &file) const;
	void loadParameters(std::ifstream &file);

protected:
	struct Parameter
	{
		GUILabel	*label;
		GUITrackbar	*trackbar;
		float		*value;
	};

	void valueChanged(GUIWindow *sender, float value);
	static void saveParameter(const std::string &name, const Parameter &value, std::ofstream &file);

	unsigned							m_nbParameters;
	HashMap<std::string, Parameter, 32>	m_parameters;
};

#endif