#ifndef H_DELEGATE
#define H_DELEGATE

class GUIWindow;

template <class Param>
class IDelegate
{
public:
	virtual ~IDelegate();

	virtual void operator ()(GUIWindow *sender, Param p) const =0;
};

template <>
class IDelegate<void>
{
public:
	virtual ~IDelegate();

	virtual void operator ()(GUIWindow *sender) const =0;
};

template <class T, class Param>
class Delegate: public IDelegate<Param>
{
	typedef void (T::*PtrFuncType)(GUIWindow *, Param);

public:
	Delegate(T *classPtr, PtrFuncType ptrFunc);
	virtual ~Delegate();

	virtual void operator ()(GUIWindow *sender, Param p) const;

private:
	T				*m_classPtr;
	PtrFuncType		 m_ptrFunc;
};

template <class T>
class Delegate<T, void>: public IDelegate<void>
{
	typedef void (T::*PtrFuncType)(GUIWindow *);

public:
	Delegate(T *classPtr, PtrFuncType ptrFunc);
	virtual ~Delegate();

	virtual void operator ()(GUIWindow *sender) const;

private:
	T				*m_classPtr;
	PtrFuncType		 m_ptrFunc;
};

template <class T, class Param>
class BlindDelegate: public IDelegate<Param>
{
	typedef void (T::*PtrFuncType)(Param );

public:
	BlindDelegate(T *classPtr, PtrFuncType ptrFunc);
	virtual ~BlindDelegate();

	virtual void operator ()(GUIWindow *, Param p) const;

private:
	T				*m_classPtr;
	PtrFuncType		 m_ptrFunc;
};

template <class T>
class BlindDelegate<T, void>: public IDelegate<void>
{
	typedef void (T::*PtrFuncType)();

public:
	BlindDelegate(T *classPtr, PtrFuncType ptrFunc);
	virtual ~BlindDelegate();

	virtual void operator ()(GUIWindow *) const;

private:
	T				*m_classPtr;
	PtrFuncType		 m_ptrFunc;
};

#include "Delegate.inl"

#endif