#ifndef H_DENSITY
#define H_DENSITY

#include "Image.h"
#include "Buffer.h"
#include <vector>
#include <d3dx9.h>

class Environment;
class DynamicTexture;
class Renderer;
class MediaManager;
class Shader;
class Declaration;

enum ECenterType
{
	CENTER_BASIC,
	CENTER_RASTER,
	CENTER_RADIAL
};

struct Center
{
	D3DXVECTOR2 pos;
	float		innerRadius;
	float		outerRadius;
	ECenterType type;
};

class Density
{
public:
	Density(MediaManager &media, Environment &env);
	~Density(void);

	void update();
	void updateCenters();
	void render(Renderer &renderer) const;
	void reset();
	/* displayCenters
	Affiche ou non les centres, selon l'argument pass */
	void displayCenters(bool disp);

	/* getters et setters */
	int getGridSize() const;
	const Image<float>& getDensityMap() const;
	Image<unsigned short>& getTextureMap();
	Image<float>& getDensityMap();
	void addCenter(const Center &toAdd);
	void addCenter(const D3DXVECTOR2& pos, const float innerRadius, const float outerRadius, ECenterType type);
	std::vector<Center>& getCenters();
	const std::vector<Center> & getCenters() const;
	const Environment& getEnvironment() const;
	Environment	& getEnvironment();

private:
	Environment				*m_env;
	int						 m_gridSize;
	Image<float>			 m_densityMap;
	std::vector<Center>		 m_centers;

	// variables graphiques
	Image<unsigned short>				 m_textureMap;
	DynamicTexture						*m_texture;
	MediaManager						*m_media;
	Shader								*m_shader;
	Shader								*m_shaderCenter;
	Declaration							*m_declaration;
	VertexBuffer<D3DXVECTOR3>::Type		*m_vbCenters;
	IndexBuffer<unsigned short>::Type	*m_ibCenters;

	// fonctions prives
	const float getLocalDensity(const D3DXVECTOR2& pos) const;
};

#endif