#ifndef H_ENVIRONMENT
#define H_ENVIRONMENT

#include <d3dx9.h>
#include "Buffer.h"
#include "Image.h"

#define GRID_PRECISION 7

#if GRID_PRECISION < 8
#define INDEX_TYPE unsigned short
#else
#define INDEX_TYPE unsigned int
#endif

class MediaManager;
class Declaration;
class Renderer;
class Shader;

class Environment
{
public:
	Environment(MediaManager &media, int gridPrecision, float waterLevel = -2.0f);
	~Environment(void);

	/* render
	Fonctions de rendu. */
	void render(Renderer &renderer) const;
	void renderLandscape(Renderer &renderer) const;
	void renderWater(Renderer &renderer) const;
	/* synchroHeights
	Synchronise les hauteurs entre la height map et le vertex buffer */
	void synchroHeights();

	/* Rinitialise l'environnement */
	void reset();

	/* Getters et Setters */
	int getGridSize() const;
	int getGridPrecision() const;
	const Image<float> & getHeightMap() const;
	Image<float> & getHeightMap();
	float getWaterLevel() const;
	float & getWaterLevel();
	void setWaterLevel(float waterLevel);
	void setMinHeight(float minHeight);
	void setMaxHeight(float maxHeight);
	float getMinHeight() const;
	float getMaxHeight() const;

protected:
	float computeDX(int x1, int x3, int y) const;
	float computeDY(int y1, int y3, int x) const;

private:
	int m_gridPrecision,
		m_gridSize;
	float	m_waterLevel,
			m_minHeight,
			m_maxHeight;
	Image<float> m_heightsMap;

	/* variables d'affichage */
	D3DXVECTOR2	pos;

	VertexBuffer<D3DXVECTOR2>::Type		*m_vbLandscape;
	VertexBuffer<D3DXVECTOR4>::Type		*m_normals;
	IndexBuffer<INDEX_TYPE>::Type		*m_ibLandscape;
	Declaration							*m_declLandscape;
	VertexBuffer<D3DXVECTOR2>::Type		*m_vbWater;
	Declaration							*m_declWater;
	Shader								*m_shaderWater;
	Shader								*m_shaderLandscape;
};

#endif
