#ifndef H_GUI
#define H_GUI

#include "Listener.h"
#include "Buffer.h"
#include <d3dx9.h>

class GUIDesktopWindow;
class GUIWindow;
class GUITheme;
class Renderer;
class MediaManager;
class Font;
class Texture;
class Declaration;
class Shader;

struct GUIVertex
{
	D3DXVECTOR2	position;
	D3DXVECTOR2 texcoord;
};

class Appli;

class GUI: public Listener
{
public:
	GUI(IOManager &io, MediaManager &media, Appli &appli);
	~GUI();

	virtual void onKeyDown(const IOManager &io, unsigned key, float elapsedTime);
	virtual void onKeyPressed(const IOManager &io, unsigned key);
	virtual void onKeyReleased(const IOManager &io, unsigned key);
	virtual void onKeyChar(const IOManager &io, unsigned key);
	virtual void onMouseMove(const IOManager &io, int dx, int dy);
	virtual void onMouseDrag(const IOManager &io, int dx, int dy);
	virtual void onMouseClick(const IOManager &io, unsigned button);
	virtual void onMouseRelease(const IOManager &io, unsigned button);
	virtual void onMouseWheel(const IOManager &io, float delta);
	void onSize(unsigned width, unsigned height);

	void setFocus(GUIWindow *focusWindow);

	void update(float elapsedTime);
	void render(Renderer &renderer) const;

	void drawRect(Renderer &renderer, VertexBuffer<GUIVertex>::Type *vb, Texture *tex, float alpha = 1.0f) const;

	GUITheme * getTheme() const;
	GUIWindow * getRoot() const;
	void addWindow(GUIWindow *window);

	void captureMouse(GUIWindow *window);
	void releaseMouse(GUIWindow *window);

	// TODO: temporaire
	MediaManager & getMedia() const;

private:
	GUIDesktopWindow					*m_root;
	GUIWindow							*m_focus;
	GUIWindow							*m_captureMouse;
	GUITheme							*m_theme;
	MediaManager						*m_media;
	IndexBuffer<unsigned short>::Type	*m_ib;
	Declaration							*m_decl;
	Shader								*m_shader;
};

#endif