#include "GUIButton.h"
#include "GUITexture.h"
#include "GUILabel.h"
#include "GUITheme.h"
#include "MemoryLog.h"

enum EButtonTexture
{
	BUTTON_TEXTURE_UP = 0,
	BUTTON_TEXTURE_DOWN,
	BUTTON_TEXTURE_HIGHLIGHT,
	BUTTON_TEXTURE_HIGHTLIGHT_DOWN
};

GUIButton::GUIButton(GUI *gui, const std::string &name, GUIWindow *parent, const std::string &caption):
GUIWindow(gui, name, parent),
m_togglable(false),
m_pressed(false),
m_actTexture(0)
{
	GUILabel *l = new GUILabel(gui, name + ".Caption", this, caption, gui->getTheme()->getButtonFont());
	l->getPosition().setPosition(D3DXVECTOR2(0.0f, 0.0f), GA_CENTER, GA_CENTER);

	m_textures[BUTTON_TEXTURE_UP] = new GUITexture(gui->getMedia(), "Button_up.png", *this);
	m_textures[BUTTON_TEXTURE_UP]->getSize().setRelativeSize(D3DXVECTOR2(0.0f, 0.0f), D3DXVECTOR2(1.0f, 1.0f));

	m_textures[BUTTON_TEXTURE_DOWN] = new GUITexture(gui->getMedia(), "Button_down.png", *this);
	m_textures[BUTTON_TEXTURE_DOWN]->getSize().setRelativeSize(D3DXVECTOR2(0.0f, 0.0f), D3DXVECTOR2(1.0f, 1.0f));

	m_textures[BUTTON_TEXTURE_HIGHLIGHT] = new GUITexture(gui->getMedia(), "Button_highlight.png", *this);
	m_textures[BUTTON_TEXTURE_HIGHLIGHT]->getSize().setRelativeSize(D3DXVECTOR2(0.0f, 0.0f), D3DXVECTOR2(1.0f, 1.0f));

	m_textures[BUTTON_TEXTURE_HIGHTLIGHT_DOWN] = new GUITexture(gui->getMedia(), "Button_dhighlight.png", *this);
	m_textures[BUTTON_TEXTURE_HIGHTLIGHT_DOWN]->getSize().setRelativeSize(D3DXVECTOR2(0.0f, 0.0f), D3DXVECTOR2(1.0f, 1.0f));
}

GUIButton::~GUIButton()
{
	for(unsigned i = 0; i < 4; ++i)
	{
		delete m_textures[i];
	}
}

void GUIButton::setTogglable(bool togglable)
{
	m_togglable = togglable;
}

bool GUIButton::isPressed() const
{
	return m_pressed;
}

void GUIButton::renderSelf(Renderer &renderer) const
{
	m_textures[m_actTexture]->render(renderer, *getGUI());
}

void GUIButton::onMouseEnter(const D3DXVECTOR2 &, const D3DXVECTOR2 &)
{
	m_actTexture = m_pressed ? BUTTON_TEXTURE_HIGHTLIGHT_DOWN : BUTTON_TEXTURE_HIGHLIGHT;
}

void GUIButton::onMouseLeave(const D3DXVECTOR2 &, const D3DXVECTOR2 &)
{
	m_actTexture = m_pressed ? BUTTON_TEXTURE_DOWN : BUTTON_TEXTURE_UP;
}

bool GUIButton::onMouseClick(const D3DXVECTOR2 &, unsigned button)
{
	if(button == MK_LBUTTON)
	{
		if(m_togglable)
		{
			m_pressed = !m_pressed;
			m_actTexture = m_pressed ? BUTTON_TEXTURE_HIGHTLIGHT_DOWN : BUTTON_TEXTURE_HIGHLIGHT;
			onPress(this, m_pressed);
		}
		else
		{
			m_pressed = true;
			m_actTexture = BUTTON_TEXTURE_HIGHTLIGHT_DOWN;
			onClick(this);
		}
		return true;
	}
	return false;
}

bool GUIButton::onMouseRelease(const D3DXVECTOR2 &, unsigned button)
{
	if(button == MK_LBUTTON)
	{
		if(!m_togglable)
		{
			m_pressed = false;
			m_actTexture = BUTTON_TEXTURE_HIGHLIGHT;
			onRelease(this);
		}
		return true;
	}
	return false;
}