#include "GUIDesktopWindow.h"
#include "Appli.h"
#include "MemoryLog.h"

#define WHEEL_SPEED 0.02f
#define ROTATE_SPEED 0.002f
#define TRANSLATE_KEY_SPEED	32.0f
#define TRANSLATE_MOUSE_SPEED 0.1f

GUIDesktopWindow::GUIDesktopWindow(GUI *gui, Appli *application, unsigned width, unsigned height):
GUIWindow(gui, "Root", NULL),
m_application(application)
{
	getSize().setAbsoluteSize(D3DXVECTOR2(static_cast<float>(width), static_cast<float>(height)));
	getPosition().setPosition(D3DXVECTOR2(0.0f, 0.0f), GA_NONE, GA_TOPLEFT);
}

GUIDesktopWindow::~GUIDesktopWindow()
{
}

void GUIDesktopWindow::setResolution(unsigned width, unsigned height)
{
	getSize().setAbsoluteSize(D3DXVECTOR2(static_cast<float>(width), static_cast<float>(height)));
}

bool GUIDesktopWindow::onKeyPressed(unsigned key)
{
	switch(key)
	{
	case VK_ESCAPE:
		{
			GUIMessageBox *m = new GUIMessageBox(getGUI(), "Confirmation", "Voulez vous quitter?", MBT_YESNO);
			m->onClose += new Delegate<GUIDesktopWindow, const GUIMessageBox::ResultEventArg &>(
				this, &GUIDesktopWindow::onConfirmExit);
			return true;
		}
	case 'F':
		m_application->toggleFullscreen();
		return true;
	default:
		return false;
	}
}

bool GUIDesktopWindow::onKeyDown(unsigned key, float elapsedTime)
{
	switch(key)
	{
	case VK_UP: 
		m_application->getCamera().translateY(-elapsedTime * TRANSLATE_KEY_SPEED);
		m_application->getCamera().update();
		return true;
	case VK_DOWN:
		m_application->getCamera().translateY(elapsedTime * TRANSLATE_KEY_SPEED);
		m_application->getCamera().update();
		return true;
	case VK_LEFT:
		m_application->getCamera().translateX(-elapsedTime * TRANSLATE_KEY_SPEED);
		m_application->getCamera().update();
		return true;
	case VK_RIGHT:
		m_application->getCamera().translateX(elapsedTime * TRANSLATE_KEY_SPEED);
		m_application->getCamera().update();
		return true;
	default:
		return false;
	}
}

bool GUIDesktopWindow::onMouseDrag(const D3DXVECTOR2 &lastPos, const D3DXVECTOR2 &pos, unsigned buttons)
{
	bool left = (buttons & MK_LBUTTON) != 0;
	bool right = (buttons & MK_RBUTTON) != 0;

	float dx = pos.x - lastPos.x;
	float dy = pos.y - lastPos.y;

	if(left && right)
	{
		m_application->getCamera().translateX(dx * TRANSLATE_MOUSE_SPEED);
		m_application->getCamera().translateZ(dy * TRANSLATE_MOUSE_SPEED);
		m_application->getCamera().update();
		return true;
	}
	else if(left)
	{
		m_application->getCamera().rotateY(dx * ROTATE_SPEED);
		m_application->getCamera().rotateX(dy * ROTATE_SPEED);
		m_application->getCamera().update();
		return true;
	}
	else if(right)
	{
		m_application->getCamera().translateX(dx * TRANSLATE_MOUSE_SPEED);
		m_application->getCamera().translateY(dy * TRANSLATE_MOUSE_SPEED);
		m_application->getCamera().update();
		return true;
	}
	return false;
}

bool GUIDesktopWindow::onMouseWheel(const D3DXVECTOR2 &, float delta)
{
	m_application->getCamera().translateZ(delta * WHEEL_SPEED);
	m_application->getCamera().update();
	return true;
}

void GUIDesktopWindow::onConfirmExit(GUIWindow *, const GUIMessageBox::ResultEventArg &arg)
{
	if(arg.getButton() == MBB_YES)
	{
		m_application->exit();
	}
}