#include "GUIFrame.h"
#include "GUITexture.h"
#include "GUILabel.h"
#include "utils.h"
#include "GUITheme.h"
#include "MemoryLog.h"

GUIFrame::GUIFrame(GUI *gui, const std::string &name, GUIWindow *parent, const std::string &title):
GUIWindow(gui, name, parent),
m_clientSize(&getSize()),
m_clientPosition(&getPosition(), m_clientSize)
{
	setType(FT_FLOAT);

	m_labelTitle = new GUILabel(gui, name + ".LabelTitle", this, title, gui->getTheme()->getFrameHeaderFont());
	m_labelTitle->getPosition().setPosition(D3DXVECTOR2(8.0f, -4.0f), GA_TOPLEFT, GA_BOTTOMLEFT);

	m_background = new GUITexture(gui->getMedia(), "Frame_background.png", *this);
	m_background->getSize().setRelativeSize(D3DXVECTOR2(0.0f, 0.0f), D3DXVECTOR2(1.0f, 1.0f));
	m_background->setAlpha(0.8f);

	m_edges[0] = new GUITexture(gui->getMedia(), "Frame_top.png", *this);
	m_edges[0]->getSize().setRelativeSize(D3DXVECTOR2(0.0f, 24.0f), D3DXVECTOR2(1.0f, 0.0f));
	m_edges[0]->getPosition().setPosition(D3DXVECTOR2(0.0f, 0.0f), GA_TOP, GA_BOTTOM);

	m_edges[1] = new GUITexture(gui->getMedia(), "Frame_side.png", *this);
	m_edges[1]->getSize().setRelativeSize(D3DXVECTOR2(8.0f, 0.0f), D3DXVECTOR2(0.0f, 1.0f));
	m_edges[1]->getPosition().setPosition(D3DXVECTOR2(0.0f, 0.0f), GA_RIGHT, GA_LEFT);

	m_edges[2] = new GUITexture(gui->getMedia(), "Frame_side.png", *this);
	m_edges[2]->getSize().setRelativeSize(D3DXVECTOR2(8.0f, 0.0f), D3DXVECTOR2(0.0f, 1.0f));
	m_edges[2]->getPosition().setPosition(D3DXVECTOR2(0.0f, 0.0f), GA_LEFT, GA_RIGHT);

	m_edges[3] = new GUITexture(gui->getMedia(), "Frame_bottom.png", *this);
	m_edges[3]->getSize().setRelativeSize(D3DXVECTOR2(0.0f, 8.0f), D3DXVECTOR2(1.0f, 0.0f));
	m_edges[3]->getPosition().setPosition(D3DXVECTOR2(0.0f, 0.0f), GA_BOTTOM, GA_TOP);

	m_edges[4] = new GUITexture(gui->getMedia(), "Frame_rbcorner.png", *this);
	m_edges[4]->getSize().setAbsoluteSize(D3DXVECTOR2(8.0f, 8.0f));
	m_edges[4]->getPosition().setPosition(D3DXVECTOR2(0.0f, 0.0f), GA_BOTTOMRIGHT, GA_TOPLEFT);

	m_edges[5] = new GUITexture(gui->getMedia(), "Frame_lbcorner.png", *this);
	m_edges[5]->getSize().setAbsoluteSize(D3DXVECTOR2(8.0f, 8.0f));
	m_edges[5]->getPosition().setPosition(D3DXVECTOR2(0.0f, 0.0f), GA_BOTTOMLEFT, GA_TOPRIGHT);

	m_edges[6] = new GUITexture(gui->getMedia(), "Frame_ltcorner.png", *this);
	m_edges[6]->getSize().setAbsoluteSize(D3DXVECTOR2(8.0f, 24.0f));
	m_edges[6]->getPosition().setPosition(D3DXVECTOR2(0.0f, 0.0f), GA_TOPLEFT, GA_BOTTOMRIGHT);

	m_edges[7] = new GUITexture(gui->getMedia(), "Frame_rtcorner.png", *this);
	m_edges[7]->getSize().setAbsoluteSize(D3DXVECTOR2(8.0f, 24.0f));
	m_edges[7]->getPosition().setPosition(D3DXVECTOR2(0.0f, 0.0f), GA_TOPRIGHT, GA_BOTTOMLEFT);
}

GUIFrame::~GUIFrame()
{
	delete m_background;

	for(unsigned i = 0; i < 8; ++i)
	{
		delete m_edges[i];
	}
}

Position & GUIFrame::getClientPosition()
{
	return m_clientPosition;
}

Size & GUIFrame::getClientSize()
{
	return m_clientSize;
}

const Position & GUIFrame::getClientPosition() const
{
	return m_clientPosition;
}

const Size & GUIFrame::getClientSize() const
{
	return m_clientSize;
}

D3DXVECTOR2 GUIFrame::clientToWindowSize(const D3DXVECTOR2 &size)
{
	switch(m_type)
	{
	default:
	case FT_FLOAT:
		return size + D3DXVECTOR2(16.0f, 32.0f);
	case FT_DOCKLEFT:
		return size + D3DXVECTOR2(8.0f, 24.0f);
	case FT_DOCKRIGHT:
		return size + D3DXVECTOR2(8.0f, 24.0f);
	case FT_DOCKTOP:
		return size + D3DXVECTOR2(0.0f, 32.0f);
	case FT_DOCKBOTTOM:
		return size + D3DXVECTOR2(0.0f, 24.0f);
	}
}

void GUIFrame::setType(EFloatType type)
{
	switch(type)
	{
	case FT_FLOAT:
		m_clientSize.setRelativeSize(D3DXVECTOR2(-16.0f, -32.0f), D3DXVECTOR2(1.0f, 1.0f));
		m_clientPosition.setPosition(D3DXVECTOR2(8.0f, 24.0f));
		break;
	case FT_DOCKLEFT:
		m_clientSize.setRelativeSize(D3DXVECTOR2(-8.0f, -24.0f), D3DXVECTOR2(1.0f, 1.0f));
		m_clientPosition.setPosition(D3DXVECTOR2(0.0f, 24.0f));
		getPosition().setPosition(D3DXVECTOR2(0.0f, 0.0f), GA_LEFT, GA_LEFT);
		getSize().setRelativeSize(D3DXVECTOR2(0.0f, 0.0f), D3DXVECTOR2(0.25f, 1.0f));
		break;
	case FT_DOCKRIGHT:
		m_clientSize.setRelativeSize(D3DXVECTOR2(-8.0f, -24.0f), D3DXVECTOR2(1.0f, 1.0f));
		m_clientPosition.setPosition(D3DXVECTOR2(8.0f, 24.0f));
		getPosition().setPosition(D3DXVECTOR2(0.0f, 0.0f), GA_RIGHT, GA_RIGHT);
		getSize().setRelativeSize(D3DXVECTOR2(0.0f, 0.0f), D3DXVECTOR2(0.25f, 1.0f));
		break;
	case FT_DOCKTOP:
		m_clientSize.setRelativeSize(D3DXVECTOR2(0.0f, -32.0f), D3DXVECTOR2(1.0f, 1.0f));
		m_clientPosition.setPosition(D3DXVECTOR2(0.0f, 24.0f));
		getPosition().setPosition(D3DXVECTOR2(0.0f, 0.0f), GA_TOP, GA_TOP);
		getSize().setRelativeSize(D3DXVECTOR2(0.0f, 0.0f), D3DXVECTOR2(1.0f, 0.25f));
		break;
	case FT_DOCKBOTTOM:
		m_clientSize.setRelativeSize(D3DXVECTOR2(0.0f, -24.0f), D3DXVECTOR2(1.0f, 1.0f));
		m_clientPosition.setPosition(D3DXVECTOR2(0.0f, 24.0f));
		getPosition().setPosition(D3DXVECTOR2(0.0f, 0.0f), GA_BOTTOM, GA_BOTTOM);
		getSize().setRelativeSize(D3DXVECTOR2(0.0f, 0.0f), D3DXVECTOR2(1.0f, 0.25f));
		break;
	}
	m_type = type;
}

void GUIFrame::renderSelf(Renderer &renderer) const
{
	m_background->render(renderer, *getGUI());

	switch(m_type)
	{
	case FT_FLOAT:
		for(unsigned i = 0; i < 8; ++i)
		{
			m_edges[i]->render(renderer, *getGUI());
		}
		break;
	case FT_DOCKLEFT:
		m_edges[0]->render(renderer, *getGUI());
		m_edges[1]->render(renderer, *getGUI());
		m_edges[7]->render(renderer, *getGUI());
		break;
	case FT_DOCKRIGHT:
		m_edges[0]->render(renderer, *getGUI());
		m_edges[2]->render(renderer, *getGUI());
		m_edges[6]->render(renderer, *getGUI());
		break;
	case FT_DOCKTOP:
		m_edges[0]->render(renderer, *getGUI());
		m_edges[3]->render(renderer, *getGUI());
		break;
	case FT_DOCKBOTTOM:
		m_edges[0]->render(renderer, *getGUI());
		break;
	}
}
bool GUIFrame::onMouseDrag(const D3DXVECTOR2 &lastPos, const D3DXVECTOR2 &pos, unsigned buttons)
{
	if(isInHeader(lastPos) && buttons == MK_LBUTTON)
	{
		if(m_type == FT_FLOAT)
		{
			getPosition().move(pos - lastPos);
		}
	}

	return true;
}

bool GUIFrame::isInHeader(const D3DXVECTOR2 &point) const
{
	const D3DXVECTOR2 &min = m_edges[0]->getPosition().getAbsolutePosition();

	return isInRect(min, min + m_edges[0]->getSize().getAbsoluteSize(), point);
}

void GUIFrame::setTitle(const std::string &title)
{
	m_labelTitle->setText(title);
}