#include "GUILabel.h"
#include "Renderer.h"
#include "GUI.h"
#include "GUITheme.h"
#include "MemoryLog.h"

GUILabel::GUILabel(GUI *gui, const std::string &name, GUIWindow *parent, const std::string &text, Font *font):
GUIWindow(gui, name, parent),
m_text(text),
m_font(font),
m_color(1.0f, 1.0f, 1.0f, 1.0f)
{
	if(!m_font)
	{
		m_font = gui->getTheme()->getDefaultFont();
	}
	updateSize();
}

GUILabel::~GUILabel()
{
}

void GUILabel::setText(const std::string &text)
{
	m_text = text;
	updateSize();
}

const std::string & GUILabel::getText() const
{
	return m_text;
}

void GUILabel::setColor(const D3DXVECTOR4 &color)
{
	m_color = color;
}

void GUILabel::setFont(Font *font)
{
	m_font = font;
}

void GUILabel::updateSize()
{
	D3DXVECTOR2 size;
	m_font->getTextSize(m_text, size);
	getSize().setAbsoluteSize(size);
}

bool GUILabel::onMouseClick(const D3DXVECTOR2 &, unsigned button)
{
	if(button == MK_LBUTTON && onClick.hasDelegate())
	{
		onClick(this);
		return true;
	}
	return false;
}

void GUILabel::renderSelf(Renderer &renderer) const
{
	renderer.drawText(m_text, getPosition().getAbsolutePosition(), *m_font, m_color);
}