#include "GUIListBox.h"
#include "GUITexture.h"
#include "GUILabel.h"
#include "GUITheme.h"
#include "Debug.h"
#include "MemoryLog.h"

GUIListBox::Element::Element(GUILabel *label, bool active):
m_label(label),
m_active(active)
{
}

GUIListBox::GUIListBox(GUI *gui, const std::string &name, GUIWindow *parent):
GUIWindow(gui, name, parent),
m_selectedItem(-1)
{
	m_background = new GUITexture(gui->getMedia(), "ListBox_background.png", *this);
	m_background->getSize().setRelativeSize(D3DXVECTOR2(0.0f, 0.0f), D3DXVECTOR2(1.0f, 1.0f));
	m_background->setAlpha(0.8f);

	m_highlight = new GUITexture(gui->getMedia(), "ListBox_highlight.png", *this);
	m_highlight->getSize().setRelativeSize(D3DXVECTOR2(-8.0f, 24.0f), D3DXVECTOR2(1.0, 0.0f));
	m_highlight->setAlpha(0.7f);
}

GUIListBox::~GUIListBox()
{
	delete m_background;
	delete m_highlight;
}

unsigned GUIListBox::getNbElements() const
{
	return m_elems.size();
}

void GUIListBox::addElement(const std::string &label)
{
	GUILabel *guiLabel = new GUILabel(getGUI(), "elem", this, label, getGUI()->getTheme()->getListElementFont());
	guiLabel->setColor(getGUI()->getTheme()->getActiveFontColor());

	float size = guiLabel->getSize().getAbsoluteSize().y + 6.0f;
	guiLabel->getPosition().setPosition(D3DXVECTOR2(0.0f, 6.0f + size * m_elems.size()), GA_TOP, GA_TOP);

	guiLabel->onClick += new Delegate<GUIListBox, void>(this, &GUIListBox::onLabelClick);

	m_elems.push_back(Element(guiLabel, true));
}

void GUIListBox::setActive(int index, bool active)
{
	Assert(index >= 0 && static_cast<unsigned>(index) < m_elems.size());

	m_elems[index].m_active = active;
	m_elems[index].m_label->setColor(
		active ? getGUI()->getTheme()->getActiveFontColor() : getGUI()->getTheme()->getInactiveFontColor());

	if(!active && index == m_selectedItem)
	{
		m_selectedItem = -1;
		onItemSelected(this, -1);
	}
}

int GUIListBox::getSelectedItem() const
{
	return m_selectedItem;
}

void GUIListBox::select(int i)
{
	m_selectedItem = i;

	if(i >= 0)
	{
		float size = m_elems[i].m_label->getSize().getAbsoluteSize().y + 6.0f;
		m_highlight->getPosition().setPosition(D3DXVECTOR2(0.0f, 3.0f + size * i + size * 0.5f), GA_TOP, GA_CENTER);
	}
	onItemSelected(this, i);
}

void GUIListBox::autoSize()
{
	if(m_elems.empty())
	{
		getSize().setAbsoluteSizeY(12.0f);
	}
	else
	{
		getSize().setAbsoluteSizeY(6.0f + m_elems.size() * (m_elems[0].m_label->getSize().getAbsoluteSize().y + 6.0f));
	}
}

void GUIListBox::renderSelf(Renderer &renderer) const
{
	m_background->render(renderer, *getGUI());

	if(m_selectedItem >= 0)
	{
		m_highlight->render(renderer, *getGUI());
	}
}

void GUIListBox::onLabelClick(GUIWindow *sender)
{
	for(unsigned i = 0; i < m_elems.size(); ++i)
	{
		if(m_elems[i].m_label == sender)
		{
			if(m_elems[i].m_active)
			{
				select(i);
			}
		}
	}
}