#include "GuiTexture.h"
#include "MediaManager.h"
#include "Renderer.h"
#include "GUI.h"
#include "GuiWindow.h"
#include "MemoryLog.h"

GUITexture::GUITexture(MediaManager &media, const std::string &textureName, const GUIWindow &owner):
m_size(&owner.getClientSize()),
m_pos(&owner.getClientPosition(), m_size),
m_texture(NULL),
m_vb(NULL),
m_alpha(1.0f)
{
	m_texture = media.getObject<Texture>(textureName);

	m_vb = media.getRenderer().createVertexBuffer<GUIVertex>(4);
}

GUITexture::~GUITexture()
{
	delete m_vb;
}

void GUITexture::render(Renderer &renderer, const GUI &gui)
{
	const D3DXVECTOR2 &pos = m_pos.getAbsolutePosition();
	const D3DXVECTOR2 &size = m_size.getAbsoluteSize();

	GUIVertex *data = m_vb->lock();

	if(data)
	{
		data[0].position = pos;
		data[0].texcoord = D3DXVECTOR2(0.0f, 0.0f);

		data[1].position = D3DXVECTOR2(pos.x + size.x, pos.y);
		data[1].texcoord = D3DXVECTOR2(1.0f, 0.0f);

		data[2].position = D3DXVECTOR2(pos.x, pos.y + size.y);
		data[2].texcoord = D3DXVECTOR2(0.0f, 1.0f);

		data[3].position = pos + size;
		data[3].texcoord = D3DXVECTOR2(1.0f, 1.0f);

		m_vb->unlock();
	}

	gui.drawRect(renderer, m_vb, m_texture, m_alpha);
}

const Size & GUITexture::getSize() const
{
	return m_size;
}

Size & GUITexture::getSize()
{
	return m_size;
}

const Position & GUITexture::getPosition() const
{
	return m_pos;
}

Position & GUITexture::getPosition()
{
	return m_pos;
}

void GUITexture::setAlpha(float alpha)
{
	m_alpha = alpha;
}