#ifndef H_GUIWINDOW
#define H_GUIWINDOW

#include "Size.h"
#include "Position.h"
#include <vector>
#include <string>

class GUI;
class Renderer;
class GUILabel;

class GUIWindow
{
public:
	GUIWindow(GUI *gui, const std::string &name, GUIWindow *parent = NULL);
	virtual ~GUIWindow();

	GUIWindow * getParent() const;
	const Size & getSize() const;
	Size & getSize();
	const Position & getPosition() const;
	Position & getPosition();
	const std::string &getName() const;
	GUIWindow * getChildrenByName(const std::string &name);
	const GUIWindow * getChildrenByName(const std::string &name) const;

	virtual Position & getClientPosition();
	virtual Size & getClientSize();
	virtual const Position & getClientPosition() const;
	virtual const Size & getClientSize() const;
	virtual D3DXVECTOR2 clientToWindowSize(const D3DXVECTOR2 &size);

	void addChild(GUIWindow *child);
	void removeChild(GUIWindow *child);

	virtual bool onMouseClick(const D3DXVECTOR2 &pos, unsigned button);
	virtual bool onMouseRelease(const D3DXVECTOR2 &pos, unsigned button);
	virtual void onMouseMove(const D3DXVECTOR2 &lastPos, const D3DXVECTOR2 &pos);
	virtual void onMouseEnter(const D3DXVECTOR2 &lastPos, const D3DXVECTOR2 &pos);
	virtual void onMouseLeave(const D3DXVECTOR2 &lastPos, const D3DXVECTOR2 &pos);
	virtual bool onMouseDrag(const D3DXVECTOR2 &lastPos, const D3DXVECTOR2 &pos, unsigned buttons);
	virtual bool onMouseWheel(const D3DXVECTOR2 &pos, float delta);
	virtual bool onKeyDown(unsigned key, float elapsedTime);
	virtual bool onKeyPressed(unsigned key);
	virtual bool onKeyReleased(unsigned key);
	virtual bool onChar(unsigned key);

	bool dispatchMouseButton(const D3DXVECTOR2 &pos, unsigned button, bool down);
	bool dispatchMouseWheel(const D3DXVECTOR2 &pos, float delta);
	void dispatchMouseMove(const D3DXVECTOR2 &lastPos, const D3DXVECTOR2 &pos);
	bool dispatchMouseDrag(const D3DXVECTOR2 &lastPos, const D3DXVECTOR2 &pos, unsigned buttons);
	void dispatchKeyPress(unsigned key, bool down, bool translated);
	void dispatchKeyDown(unsigned key, float elapsedTime);

	bool isInWindow(const D3DXVECTOR2 &point) const;

	bool isVisible() const;
	void show();
	void hide();
	void close();

	void getFocus();

	void update(float elapsedTime);
	void render(Renderer &renderer) const;

	GUI * getGUI() const;

protected:
	virtual void updateSelf(float elapsedTime);
	virtual void renderSelf(Renderer &renderer) const;

private:
	GUIWindow					*m_parent;
	GUI							*m_gui;
	Size						 m_size;
	Position					 m_position;
	std::vector<GUIWindow *>	 m_children;
	std::string					 m_name;
	bool						 m_visible;
	bool						 m_closed;
};

#endif