#ifndef H_GENBLOCKS
#define H_GENBLOCKS

#include "generator.h"
#include "HashMap.h"
#include "Geometry.h"
#include <vector>
#include <d3dx9.h>

class RoadNetwork;
class Road;
class Block;
class Crossing;
class BlockSet;
class Environment;
class Density;

struct RoadMark
{
	RoadMark (bool be = false, bool eb = false)
	{
		beginEnd = be;
		endBegin = eb;
	}

	bool beginEnd, endBegin;
};

typedef HashMap<const Crossing*, std::vector<D3DXVECTOR3> > WallVertices;
typedef HashMap<const Crossing*, std::vector<unsigned short> > CrossingVertices;

class GenBlocks :
	public Generator
{
public:
	GenBlocks(RoadNetwork &roadNetwork, Geometry &roadsGeometry, BlockSet &blockSet, const Environment &env, const Density &density);
	~GenBlocks(void);
	
	virtual bool isFinished() const;
	virtual void commitChange();

protected:
	virtual void proceed();
	virtual void finish();
	virtual void createParameters(Control *control);
	virtual void attachControl(Control *control);

	void computeCrossingGeometry(Crossing &c);
	void computeCrossingVertices(Crossing &c, std::vector<Geometry::VertexStruct> *vertices);
	void computeCorner(const Road &r1, const Road &r2, const Crossing &common, D3DXVECTOR2 &sidewalk, D3DXVECTOR2 &wall) const;
	D3DXVECTOR3 computeNormal(const Road &r1, const Road &r2, const Crossing &common) const;
	void computeRoadGeometry(const Road *r);
	float getRoadWidth(const Road &r) const;
	float getSidewalkWidth(const Road &r) const;

private:
	BlockSet						*m_blockSet;
	Geometry						*m_roadsGeometry;
	const std::vector<Road*>		*m_roads;
	std::vector<Crossing *>			*m_crossings;
	HashMap<const Road*, RoadMark>	 m_roadMarks;
	CrossingVertices				 m_crossingVertices;
	WallVertices					 m_wallVertices;
	unsigned int					 m_actualRoad;
	unsigned						 m_actualCrossing;
	const Environment				*m_env;
	const Density					*m_density;

	float							 m_wayWidth;
	float							 m_sidewalkStreetWidth;
	float							 m_sidewalkHighwayWidth;
	float							 m_sidewalkHeight;

	float							 m_centerCommercialProportion;
	float							 m_outskirtCommercialProportion;
	float							 m_centerDensityTreshold;

	/* computeAdjancentBlock
	Trouve un bloc adjacent  la route donne.
	direction : sens de rotation. On tourne toujours  droite.
		true = dbut vers fin
		false = fin vers dbut
	*/
	Block* computeAdjancentBlock (const Road &r, bool direction);
	/* findRightCrossing
	Retourne le croisement au bout de la premire route  droite, en partant du croisement c de la route. r */
	const Road* findRightRoad(const Road &r, const Crossing &c) const;
	/* addPoints
	Ajoute le(s) point(s) de dcalage correspondant  common, pour l'intersection de la route r1 et r2  la liste points. */
	void addPoints(const Road &r1, const Crossing &common, std::vector<D3DXVECTOR3> &points) const;
};

#endif