#ifndef H_GEOMETRY
#define H_GEOMETRY

#include <vector>
#include "Buffer.h"
#include <d3dx9.h>

class MediaManager;
class Renderer;
class Declaration;
class Shader;

#define GEO_NB_LAYERS 3

class Geometry
{
public:
	struct VertexStruct
	{
		D3DXVECTOR3 position;
		D3DXVECTOR3 normal;
	};

	Geometry(MediaManager &media);
	~Geometry();

	unsigned short addVertices(const std::vector<VertexStruct> &vertices, unsigned layer);
	void addIndices(const std::vector<unsigned short> &indices, unsigned layer);

	void commitChange();
	void render(Renderer &renderer) const;
	void computeNormals();

	void reset();

protected:
	void recomputeBuffer(bool forceVerticesUpdate = false);

private:
	struct Layer
	{
		VertexBuffer<VertexStruct>::Type	*m_vb;
		IndexBuffer<unsigned short>::Type	*m_ib;
		std::vector<VertexStruct>			 m_vertices;
		std::vector<VertexStruct>			 m_newVertices;
		std::vector<unsigned short>			 m_indices;
		std::vector<unsigned short>			 m_newIndices;

		Layer();
		~Layer();
	};
	Shader								*m_shader;
	Declaration							*m_declaration;
	MediaManager						*m_media;
	Layer								 m_layers[GEO_NB_LAYERS];
};

#endif