#ifndef H_GRAPH
#define H_GRAPH

#include <vector>
#include <d3dx9math.h>
#include "buffer.h"

class Way;
class Junction;
class RoadNetwork;
class Crossing;
class Road;
class Intersection;
class IntersectionTrafficLight;
class MediaManager;
class Renderer;
class Shader;
class Declaration;

class Graph
{
public:
	Graph(RoadNetwork &roadNetwork, MediaManager &media);
	~Graph(void);

	const std::vector<Way*>& getWays() const;
	const std::vector<Junction*>& getJunctions()const;	
	const std::vector<Intersection*>& getIntersections() const;
	void render(Renderer &)const;

private:
	std::vector<Way*>							 m_ways;	
	std::vector<Junction*>						 m_junctions;	
	std::vector<Intersection*>					 m_intersections;
	MediaManager								*m_media;

	struct Vertex
	{
		D3DXVECTOR3 position;
		D3DXVECTOR3 normal;
		char		index[4];
	};

	struct RoadAngle
	{
		const Road *road;
		float angle;
		RoadAngle (const Road *r, float a)
		{
			road = r;
			angle = a;
		}
	};

	struct TrafficLight
	{
		Junction					*junction;
		IntersectionTrafficLight	*intersection;
	};

	std::vector<TrafficLight>			 m_trafficLights;
	Shader								*m_shader;
	Declaration							*m_declaration;
	VertexBuffer<Vertex>::Type			*m_vb;
	IndexBuffer<unsigned short>::Type	*m_ib;

	float computeRoadAngle(const Road *r, const Crossing *current);
	void computeOrderedRoadVector(const Crossing *c, std::vector<const Road*>& out);
	Junction* getPreviousJunction(Junction *j);
	D3DXVECTOR3	getJunctionPosition(const Road* r1, const Road* r2) const;
	
};

#endif