#ifndef H_HRTF
#define H_HRTF

#include <complex>
#include <string>

#define HRTF_MIN_ELEV -40
#define HRTF_MAX_ELEV 90
#define HRTF_INC_ELEV 10
#define HRTF_NB_ELEV (((HRTF_MAX_ELEV - HRTF_MIN_ELEV) / HRTF_INC_ELEV) + 1)

#define HRTF_SAMPLE_LENGTH 128

struct HRTFIndex
{
	int indexElev;
	int indexAzimuth;
	bool flipChannel;
};

class HRTF
{
public:
	HRTF();
	~HRTF();

	void load(const std::string &path);
	const std::complex<float> * get(const HRTFIndex &index, int channel) const;
	HRTFIndex getIndex(float elev, float azimuth) const;

protected:
	void loadFile(const std::string &filename, int elevIndex, int azimuthIndex);
	std::string getFileName(int elev, int azimuth) const;

private:
	std::complex<float> *m_data[HRTF_NB_ELEV];
};

extern HRTF g_HRTF;

#endif