/*****************************************************************************/
/* Name:				HashMap.h
/* Purpose:				Gestion d'une table de hashage 
/* Author:				Bonnet Benjamin
/* Created:				03/11/2008
/* Last Modification:	06/11/2008
/*****************************************************************************/

#ifndef H_HASHMAP
#define H_HASHMAP

#define HASHMAP_DEFAULT_LENGTH 512

/*****************************************************************************/
/* Classe grant une table de hachage
/* Permet de stocker des lments indexs par une cl et de les retrouver
/* en O(1).
/*	K:		Type de la cl
/*	T:		Type des lments stocks
/*	size:	Taille du tableau (doit tre une puissance de 2)
/*****************************************************************************/

template <class K, class T, int size = HASHMAP_DEFAULT_LENGTH>
class HashMap
{
public:
	// Constructeur
	// Initialise la mmoire ncessaire
	HashMap();
	// Destructeur
	~HashMap();

	// Indique s'il existe dj un lment index par la cl
	bool isExist(const K &key) const;
	// Retire l'lment index par la cl de la table de hachage
	void remove(const K &key);

	// Retire tous les lments de la table
	void clear();

	// Retourne une rfrence sur l'lment index par la cl
	T & operator[] (const K &key);

	// Retourne un pointeur sur l'lment index par la cl ou NULL si
	// aucun lment n'est index par cette cl
	T * get(const K &key) const;

	// Applique la fonction sur tous les lements de la table
	void forEach(void (*func)(T &));
	template <class Param>
	void forEach(void (*func)(const K &, T &, Param), Param param);
	template <class Param>
	void forEach(void (*func)(const K &, const T &, Param), Param param) const;

private:

	// Structure de donnes qui contient un noeud d'une liste chaine
	// contenant un lment stock et sa cl
	struct Item
	{
		T		item;	// lment stock
		K		key;	// cl
		Item	*next;	// pointeur sur le noeud suivant

		// Constructeur
		Item(const K &key, Item *next);
	};

	// Tableau de listes chaines contenant toutes les lments stocks
	Item *m_data[size];
};

#include "HashMap.inl"

#endif