#include "IOManager.h"

#include "MemoryLog.h"

IOManager::IOManager()
{
	// initialisation du tableau d'tat du clavier
	for (int i=0; i<256; i++)
	{				
		m_keyboardState[i] = false;
	}

	m_mouseState = 0;

	m_mouseX = 0;
	m_mouseY = 0;
}

IOManager::~IOManager(void)
{
}

void IOManager::update(float elapsedTime)
{
	for(unsigned i = 0; i < 256; ++i)
	{
		if(m_keyboardState[i])
		{
			std::list<Listener*> *listeners = &m_listeners[KEYDOWN];
			for(std::list<Listener*>::iterator it = listeners->begin(); it != listeners->end(); ++it)
			{
				(*it)->onKeyDown(*this, i, elapsedTime);
			}
		}
	}
}

void IOManager::injectKeyDown(unsigned key)
{
	if(!m_keyboardState[key])
	{
		std::list<Listener*> *listeners;
		listeners = &m_listeners[KEYPRESSED];
		for(std::list<Listener*>::iterator it = listeners->begin(); it != listeners->end(); ++it)
		{
			(*it)->onKeyPressed(*this, key);
		}
		m_keyboardState[key] = true;
	}
}

void IOManager::injectKeyUp(unsigned key)
{
	std::list<Listener*> *listeners;
	listeners = &m_listeners[KEYRELEASED];
	for(std::list<Listener*>::iterator it = listeners->begin(); it != listeners->end(); ++it)
	{
		(*it)->onKeyReleased(*this, key);
	}
	m_keyboardState[key] = false;
}

void IOManager::injectKeyChar(unsigned key)
{
	std::list<Listener*> *listeners;
	listeners = &m_listeners[KEYCHAR];
	for(std::list<Listener*>::iterator it = listeners->begin(); it != listeners->end(); ++it)
	{
		(*it)->onKeyChar(*this, key);
	}
}

void IOManager::injectMouseMove(int x, int y)
{
	int dx = x - m_mouseX;
	int dy = y - m_mouseY;

	m_mouseX = x;
	m_mouseY = y;

	if(m_mouseState)
	{
		std::list<Listener*> *listeners;
		listeners = &m_listeners[MOUSEDRAG];
		for(std::list<Listener*>::iterator it = listeners->begin(); it != listeners->end(); ++it)
		{
			(*it)->onMouseDrag(*this, dx, dy);
		}
	}
	else
	{
		std::list<Listener*> *listeners;
		listeners = &m_listeners[MOUSEMOVE];
		for(std::list<Listener*>::iterator it = listeners->begin(); it != listeners->end(); ++it)
		{
			(*it)->onMouseMove(*this, dx, dy);
		}
	}
}

void IOManager::injectMouseWheel(float var)
{
	std::list<Listener*> *listeners;
	listeners = &m_listeners[MOUSEWHEEL];
	for(std::list<Listener*>::iterator it = listeners->begin(); it != listeners->end(); ++it)
	{
		(*it)->onMouseWheel(*this, var);
	}
}

void IOManager::injectMouseClick(unsigned int button)
{
	m_mouseState |= button;

	std::list<Listener*> *listeners;
	listeners = &m_listeners[MOUSECLICK];
	for(std::list<Listener*>::iterator it = listeners->begin(); it != listeners->end(); ++it)
	{
		(*it)->onMouseClick(*this, button);
	}
}

void IOManager::injectMouseRelease(unsigned int button)
{
	m_mouseState &= ~button;

	std::list<Listener*> *listeners;
	listeners = &m_listeners[MOUSERELEASE];
	for(std::list<Listener*>::iterator it = listeners->begin(); it != listeners->end(); ++it)
	{
		(*it)->onMouseRelease(*this, button);
	}
}

int IOManager::getMouseX(void) const
{
	return m_mouseX;
}

int IOManager::getMouseY(void) const
{
	return m_mouseY;
}

unsigned IOManager::getMouseState() const
{
	return m_mouseState;
}

bool IOManager::isKeyPressed(unsigned key) const
{
	return m_keyboardState[key];
}

bool IOManager::isMouseButtonClicked(unsigned button) const
{
	return (m_mouseState & button) != 0;
}

void IOManager::registerListener(Listener *listener, E_INPUTTYPE input)
{
	m_listeners[input].push_back(listener);
}
