#ifndef H_IOMANAGER
#define H_IOMANAGER

#include "Listener.h"
#include <list>

enum E_INPUTTYPE
{
	KEYDOWN,
	KEYRELEASED,
	KEYPRESSED,
	KEYCHAR,
	MOUSEMOVE,
	MOUSEDRAG,
	MOUSEWHEEL,
	MOUSECLICK,
	MOUSERELEASE,

	MAX_INPUTTYPE
};

class IOManager
{
private:
	bool		m_keyboardState[256];
	unsigned	m_mouseState;	
	std::list<Listener*> m_listeners[MAX_INPUTTYPE];

	int			m_mouseX;
	int			m_mouseY;

public:
	/* Constructeur
	Prend en paramtre la scne sur laquelle il devra agir. */
	IOManager();
	/* Destructeur */
	~IOManager(void);
	/* update
	Met  jour la scne selon les appels systme reu. Principalement, met 
	jour la scne selon les informations transmises par la souris et le 
	clavier. Applique les actions correspondantes aux touches utilises.
	Prend en paramtre le temps coul (en secondes) depuis la dernire mise 
	jour. */
	void update(float timeSinceLastUpdate);

	// injectKeyDown
	// Signale qu'une touche  t appuy
	void injectKeyDown(unsigned key);
	// injectKeyUp
	// Signale qu'une touche a t relach
	void injectKeyUp(unsigned key);
	void injectKeyPressed(unsigned key);
	void injectKeyChar(unsigned key);
	void injectMouseMove(int x, int y);
    void injectMouseWheel(float var);
	void injectMouseClick(unsigned button);
	void injectMouseRelease(unsigned button);

	int getMouseX() const;
	int getMouseY() const;
	unsigned getMouseState() const;
	bool isKeyPressed(unsigned key) const;
	bool isMouseButtonClicked(unsigned button) const;

	void registerListener(Listener *listener, E_INPUTTYPE input);
};

#endif
