#include "Debug.h"
#include <cmath>
#include "MemoryLog.h"

template <class Format>
Image<Format>::Image(unsigned width, unsigned height):
m_width(width),
m_height(height)
{
	m_data = new Format[width * height];
}

template <class Format>
Image<Format>::~Image()
{
	delete[] m_data;
}

template <class Format>
const Format & Image<Format>::operator ()(int i, int j) const
{
	Assert((i >= 0) && (i < static_cast<int>(m_width)));
	Assert((j >= 0) && (j < static_cast<int>(m_height)));

	return m_data[j * m_width + i];
}

template <class Format>
Format & Image<Format>::operator ()(int i, int j)
{
	Assert((i >= 0) && (i < static_cast<int>(m_width)));
	Assert((j >= 0) && (j < static_cast<int>(m_height)));

	return m_data[j * m_width + i];
}

template <class Format>
Format lerp(const Format &p1, const Format &p2, float d)
{
	return p1 * (1.0f - d) + p2 * d;
}

template <class Format>
Format Image<Format>::interpolate(float i, float j) const
{
	int x1 = static_cast<int>(std::floor(i));
	int y1 = static_cast<int>(std::floor(j));

	float dx = i - static_cast<float>(x1);
	float dy = j - static_cast<float>(y1);

	int x2 = (x1 == static_cast<int>(m_width) - 1) ? x1 : x1 + 1;
	int y2 = (y1 == static_cast<int>(m_height) - 1) ? y1 : y1 + 1;

	const Format &f1 = m_data[y1 * m_width + x1];
	const Format &f2 = m_data[y1 * m_width + x2];
	const Format &f3 = m_data[y2 * m_width + x1];
	const Format &f4 = m_data[y2 * m_width + x2];

	return lerp(
		lerp(f1, f2, dx),
		lerp(f3, f4, dx),
		dy);
}

template <class Format>
const Format * Image<Format>::getData() const
{
	return m_data;
}

template <class Format>
unsigned Image<Format>::getWidth() const
{
	return m_width;
}

template <class Format>
unsigned Image<Format>::getHeight() const
{
	return m_height;
}

#undef new