#include "Intersection.h"

#include "Junction.h"
#include "utils.h"
#include "Way.h"
#include <d3dx9Math.h>
#include "Agent.h"
#include "MemoryLog.h"

Intersection::Intersection():
m_roadWidth(-1.0f)
{
}

Intersection::~Intersection(void)
{
}

void Intersection::addJunction(Junction& junction)
{
	m_junctions.push_back(&junction);
	// on indique  la junction son indice dans notre intersection
	junction.setIntersectionIndice(m_junctions.size() - 1);
}

void Intersection::update(float timeElapsed)
{
	// ensuite, on fait la mise  jours classique
	pUpdate(timeElapsed);
}

void Intersection::pUpdate(float)
{
}

bool Intersection::passable(const Agent&) const
{
	return true;
}

void Intersection::agentPass(const Agent &)
{
}

void Intersection::init()
{
	for (unsigned i = 0; i < m_junctions.size(); i++)
	{
		for (unsigned j = 0; j < m_junctions[i]->getWaysOut().size(); j++)
		{
			if( &m_junctions[i]->getWaysOut()[j]->getEnd().getIntersection() == this)
			{
				// la voie est interne  notre Intersection
				m_innerWays.push_back(m_junctions[i]->getWaysOut()[j]);
			}
			else
			{
				// la voie donne accs  d'autres intersections
				m_outerWays.push_back(m_junctions[i]->getWaysOut()[j]);
				m_accessTo.push_back(&m_junctions[i]->getWaysOut()[j]->getEnd().getIntersection());
			}
		}
	}
	
	if (m_junctions.size() >= 9)
	{
		// on calcule la largeur de la premire route
		D3DXVECTOR3 vLength = m_junctions[1]->getPosition() - m_junctions[4]->getPosition();
		m_roadWidth = D3DXVec3Length(&vLength);
	}
}

const std::vector<Junction*>& Intersection::getJunctions() const
{
	return m_junctions;
}

const std::vector<Way*>& Intersection::getInnerWays() const
{
	return m_innerWays;
}

const std::vector<Intersection*>& Intersection::getNextIntersections() const
{
	return m_accessTo;
}

bool Intersection::isPrioritary(const Agent& ) const
{	
	return false;
}

unsigned Intersection::getIndex()const
{
	return m_index;
}
	
void Intersection::setIndex(unsigned index)
{
	m_index = index;
}

Way* Intersection::getWayTo (const Intersection& destination) const
{
	for (unsigned i = 0; i < m_outerWays.size(); i++)
	{
		if (&m_outerWays[i]->getEnd().getIntersection() == &destination)
		{
			return m_outerWays[i];
		}
	}
	return NULL;
}

bool Intersection::enoughSpaceInDestination(const Agent& agent) const
{
	// si il n'y a pas de prochaine intersection, on est arriv  destination, on devrait disparraitre, mais on laisse passer
	if (!agent.getNextIntersection())
	{
		return true;
	}
	// on rcuppre la way entre notre intersection et la suivante (la route o l'on veut se rendre)
	const Way& way = *agent.getCurrentDestination().getIntersection().getWayTo(*agent.getNextIntersection());
	// puis on laisse passer si il y a assez de place
	const Agent* behind = way.agentBehind();
	// si il n'y a pas d'agent sur la route, ou qu'il y a la place pour l'agent, on passe
	if (	!behind
		||	way.getLength() - behind->getDistanceFromCurrentJunction() - behind->getLength()
			> agent.getLength() + agent.getSafetyDistance())
	{
		return true;
	}
	// sinon, on regarde si le dernier agent est en train d'avancer et qu'il a suffisament de place devant lui pour qu'on puisse
	// se mettre derrire
	if (behind->getCurrentSpeed() < (agent.getLength() + agent.getSafetyDistance()) / 0.7f)
	{
		return false;
	}
	const Agent* aheadBehind = way.agentAhead(*behind);
	if (!aheadBehind)
	{
		return true;
	}

	return	way.getLength() - aheadBehind->getDistanceFromCurrentJunction() - aheadBehind->getLength()
				- behind->getLength() - behind->getSafetyDistance()
			> agent.getLength() + agent.getSafetyDistance();
}