#include "IntersectionPriority.h"

#include "Agent.h"
#include "Way.h"
#include "Junction.h"
#include <algorithm>
#include "utils.h"
#include "MemoryLog.h"

IntersectionPriority::IntersectionPriority():
Intersection(),
m_main1(NULL),
m_main2(NULL)
{
}

IntersectionPriority::IntersectionPriority(const Junction& main1, const Junction& main2):
m_main1(&main1),
m_main2(&main2)
{
}

IntersectionPriority::~IntersectionPriority(void)
{
}

void IntersectionPriority::pUpdate(float)
{
}

void IntersectionPriority::init()
{
	// on appelle l'initialisation classique
	Intersection::init();
	// puis on initialise les voies prioritaires si elles n'ont pas t prcises
	if (m_main1 == NULL || m_main2 == NULL)
	{
		m_main1 = m_junctions[0];
		m_main2 = m_junctions[6];
	}
}

bool IntersectionPriority::passable(const Agent &agent) const
{
	// si on est dj engag dans le croisement, on passe
	if (agent.getCurrentDestination().getIntersectionIndice() % 3 == 2)
	{
		return true;
	}

	// on ne passe que si il y a la place
	if (!enoughSpaceInDestination(agent))
	{
		return false;
	}

	// si on est prioritaire
	if (isPrioritary(agent))
	{
		// dans tous les cas, on peut franchir l'intersection d'entre
		if (&agent.getCurrentWay()->getBegin().getIntersection() != this)
		{
			return true;
		}
		// ensuite, si on tourne  droite ou que l'on part tout droit, on peut passer
		if (	agent.getNextDestination().getIntersectionIndice() == agent.getCurrentDestination().getIntersectionIndice() + 1
			||	(	m_junctions.size() == 12
				 &&	agent.getNextDestination().getIntersectionIndice() 
					== (agent.getCurrentDestination().getIntersectionIndice() + 4) % m_junctions.size()) )
		{
			return true;
		}
		// si on tourne  gauche, on a  croiser la voie prioritaire d'en face
		unsigned faceJuncInd = (m_junctions.size() == 12)	? (agent.getCurrentDestination().getIntersectionIndice() + 6) % 12
															: (agent.getCurrentDestination().getIntersectionIndice() + 3) % 9;
		Junction& faceJunc = *m_junctions[faceJuncInd];
		// on regarde si un agent arrive ou si un agent est en train de traverser la voie que l'on doit croiser
		// (la way qui croise est toujours la way d'indice 1)
		if (	faceJunc.getWaysIn()[0]->timeBeforeNextAgent() > TIMETOPASSNEXT
			|| (	faceJunc.getWaysOut()[1]->agentBehind() 
				&&	faceJunc.getWaysOut()[1]->agentBehind()->getDistanceFromCurrentJunction() 
						< faceJunc.getWaysOut()[1]->getLength() - m_roadWidth))
		{
			return true;
		}
		// l'agent ne peut pas passer
		return false;
	}
	else // l'agent n'est pas prioritaire
	{
		// si on est  l'intrieur de l'intersection, les vrifications ont t faites, on peut circuler
		if (&agent.getCurrentWay()->getBegin().getIntersection() == this)
		{
			return true;
		}
		// sinon, on se prpare  rentrer dans l'intersection, on regarde si on peut passer
		unsigned	nextInd = agent.getNextDestination().getIntersectionIndice(),
					outInd = agent.getNextWay(2)->getEnd().getIntersectionIndice();
		
		// Vrification de la route d'en face
		if (outInd == (nextInd + 7) % m_junctions.size()) // quand on tourne  gauche
		{
			if (	m_junctions[(nextInd + 6) % m_junctions.size()]->getWaysIn()[0]->timeBeforeNextAgent() < TIMETOPASSNEXT
				|| (	m_junctions[(nextInd + 6) % m_junctions.size()]->getWaysOut()[1]->agentBehind() 
					&&	m_junctions[(nextInd + 6) % m_junctions.size()]->getWaysOut()[1]->agentBehind()->getDistanceFromCurrentJunction() 
							> m_junctions[(nextInd + 6) % m_junctions.size()]->getWaysOut()[1]->getLength() - m_roadWidth))
			{
				return false;
			}
		}

		// Vrification de la voie de droite
		if (	outInd == (nextInd + 4) % m_junctions.size()	// quand on va tout droit
			||	outInd == (nextInd + 7) % m_junctions.size())	// ou que l'on va  gauche
		{
			if (m_junctions[(nextInd + 3) % m_junctions.size()]->getWaysIn()[0]->timeBeforeNextAgent() < TIMETOPASSNEXT)
			{
				return false;
			}
		}

		// Vrification de la voie de gauche
		if (	outInd == (nextInd + 1) % m_junctions.size()	// quand on tourne  droite
			||	outInd == (nextInd + 4) % m_junctions.size()	// que l'on va tout droit
			||	outInd == (nextInd + 7) % m_junctions.size())	// ou qu'on tourne  gauche
		{		
			unsigned ind = (nextInd + m_junctions.size() - 3) % m_junctions.size();
			if (	m_junctions[ind]->getWaysIn()[0]->timeBeforeNextAgent() < TIMETOPASS
				||	(	m_junctions[ind]->getWaysOut()[1]->agentBehind()
					 &&	m_junctions[ind]->getWaysOut()[1]->agentBehind()->getDistanceFromCurrentJunction() 
							> m_junctions[ind]->getWaysOut()[1]->getLength() - m_roadWidth))
			{
				return false;
			}
		}
		// on a tout vrifi, on peut passer
		return true;
	}
}

bool IntersectionPriority::agentCrossWay(const Way& way) const
{
	// si la route n'est pas dans notre intersection, il n'y a pas de croisement
	if (	&way.getBegin().getIntersection() != this
		||	&way.getEnd().getIntersection() != this)
	{
		return false;
	}

	// on passe toutes les ways de notre intersection
	for (unsigned i = 0; i < m_innerWays.size(); i++)
	{
		// si il n'y a pas de croisement, on passe
		if (	way.isCrossed(*m_innerWays[i])
			&&	m_innerWays[i]->getAgents().size() > 0)
		{
			return true;
		}
	}
	return false;
}

bool IntersectionPriority::agentCrossJunction(const Junction &junc) const
{
	const Agent* obstacle = junc.getNearestAgentArriving();
	return	obstacle != NULL;
}

bool IntersectionPriority::isPrioritary(const Agent& agent) const
{
	return		&agent.getCurrentDestination() == m_main1 || &agent.getCurrentDestination() == m_main2
			||	&agent.getCurrentWay()->getBegin() == m_main1 || &agent.getCurrentWay()->getBegin() == m_main2;
}

bool IntersectionPriority::isIntersectionTrafficLight()const
{
	return false;
}