#include "Junction.h"

#include "Way.h"
#include "Intersection.h"
#include "Agent.h"
#include "Debug.h"
#include "MemoryLog.h"

Junction::Junction(const D3DXVECTOR3& pos, Intersection& intersection):
m_position(pos),
m_intersection(&intersection),
m_intersectionIndice(0)
{
	intersection.addJunction(*this);
}

Junction::~Junction(void)
{
}

void Junction::addWayIn(Way* in)
{
	m_waysIn.push_back(in);
}

void Junction::addWayOut(Way* out)
{
	m_waysOut.push_back(out);
}

bool Junction::passable(const Agent& agent) const
{
	return m_intersection->passable(agent);
}

void Junction::agentPass(const Agent& agent)
{
	m_intersection->agentPass(agent);
}

const std::vector<Way*>& Junction::getWaysIn() const
{
	return m_waysIn;
}

const std::vector<Way*>& Junction::getWaysOut() const
{
	return m_waysOut;
}

Intersection& Junction::getIntersection() const
{
	return *m_intersection;
}

unsigned Junction::getIntersectionIndice() const
{
	return m_intersectionIndice;
}

void Junction::setIntersectionIndice(unsigned int indice)
{
	m_intersectionIndice = indice;
}

const D3DXVECTOR3& Junction::getPosition() const
{
	return m_position;
}

const Agent* Junction::getNearestAgentArriving() const
{
	const Agent* res = NULL;
	for (unsigned i = 0; i < m_waysIn.size(); i++)
	{
		if (m_waysIn[i]->getAgents().size() > 0)
		{
			if (res && res->getDistanceFromCurrentJunction() > m_waysIn[i]->agentAhead()->getDistanceFromCurrentJunction())
			{
				res = m_waysIn[i]->agentAhead();
			}
		}
	}

	return res;
}

const Way& Junction::getWayInNotFromIntersection(const Intersection& intersection) const
{
	for (unsigned i = 0; i < m_waysIn.size(); i++)
	{
		if (&m_waysIn[i]->getBegin().getIntersection() != &intersection)
		{
			return *m_waysIn[i];
		}
	}

	Assert(true);
	return *m_waysIn[0];
}