#include "MediaManager.h"
#include "ShaderLoader.h"
#include "TextureLoader.h"
#include "FontLoader.h"
#include "SoundLoader.h"
#include "Renderer.h"
#include "MemoryLog.h"

MediaManager::MediaManager(Renderer &renderer):
ResourceManager<Shader>("./Data/Shaders/"),
ResourceManager<Texture>("./Data/Textures/"),
ResourceManager<Sound>("./Data/Sound/"),
m_renderer(renderer)
{
	ResourceManager<Shader>::registerLoader(new ShaderLoader(), ".fx");
	ResourceManager<Texture>::registerLoader(new TextureLoader(), ".bmp;.dds;.dib;.hdr;.jpg;.pfm;.png;.ppm;.tga");
	ResourceManager<Font>::registerLoader(new FontLoader(), ".fnt");
	ResourceManager<Sound>::registerLoader(new SoundLoader(), ".wav");
}

MediaManager::~MediaManager()
{
	delete &m_renderer;
}

Renderer & MediaManager::getRenderer() const
{
	return m_renderer;
}

void MediaManager::onLostDevice()
{
	ResourceManager<Shader>::onLostDevice();
	ResourceManager<Font>::onLostDevice();
}

void MediaManager::onResetDevice()
{
	ResourceManager<Shader>::onResetDevice();
	ResourceManager<Font>::onResetDevice();
}
