#ifdef _DEBUG
#undef new

#include "HashMap.h"
#include "Debug.h"
#include <cstdlib>
#include <string>
#include <fstream>
#include <iostream>
#include "MemoryLog.h"

struct MemoryBlock
{
	unsigned	size;
	std::string	file;
	int			line;
	unsigned	id;
	bool		array;
};

HashMap<void *, MemoryBlock, 2048> g_memoryMap;
unsigned g_id = 0;
unsigned g_breakId = 0xFFFFFFFF;

void * MemoryLog::alloc(unsigned size, const char *file, int line, bool array)
{
	void *ptr = malloc(size);

	MemoryBlock block;
	block.size = size;
	block.file = file;
	block.line = line;
	block.id = g_id++;
	block.array = array;

	g_memoryMap[ptr] = block;

	if(block.id == g_breakId)
	{
		__debugbreak();
	}

	return ptr;
}

void MemoryLog::free(void *ptr, bool array)
{
	if(!g_memoryMap.isExist(ptr))
	{
		::free(ptr);
		return;
	}

	const MemoryBlock &block = g_memoryMap[ptr];

	Assert(block.array == array);

	g_memoryMap.remove(ptr);
}

static void logBlock(MemoryBlock &block)
{
	std::clog << block.file << " (line " << block.line << ")" << "\n";
	std::clog << "\tId: " << block.id << " - " << block.size << " octets" << (block.array ? " (array)\n" : "\n");
}

void MemoryLog::dump(const char *filename)
{
	std::ofstream file(filename);
	std::clog.rdbuf(file.rdbuf());

	std::clog << "Begin dump\n";
	g_memoryMap.forEach(logBlock);	
	std::clog << "End dump\n";
}

void MemoryLog::breakOnId(unsigned id)
{
	g_breakId = id;
}

#endif