#include "ParametersPool.h"
#include "MemoryLog.h"

ParametersPool::ParametersPool()
{
	D3DXCreateEffectPool(&m_pool);
}

ParametersPool::~ParametersPool()
{
	m_pool->Release();
}

LPD3DXEFFECTPOOL ParametersPool::getPool() const
{
	return m_pool;
}

void ParametersPool::registerSharedParameters(LPD3DXEFFECT effect)
{
	D3DXHANDLE parameterHandle;
	D3DXPARAMETER_DESC desc;
	D3DXEFFECT_DESC effectDesc;

	effect->GetDesc(&effectDesc);
	for(unsigned i = 0; i < effectDesc.Parameters; ++i)
	{
		parameterHandle = effect->GetParameter(NULL, i);
		effect->GetParameterDesc(parameterHandle, &desc);

		if(desc.Flags & D3DX_PARAMETER_SHARED)
		{
			std::string name(desc.Name);

			if(!m_parameters.isExist(name))
			{
				m_parameters[name] = ParameterRef(effect, parameterHandle);
			}
		}
	}
}

void ParametersPool::setValue(const std::string &parameter, float value) const
{
	ParameterRef *ref = m_parameters.get(parameter);

	if(ref)
	{
		ref->m_effect->SetFloat(ref->m_handle, value);
	}
}

void ParametersPool::setValue(const std::string &parameter, const D3DXVECTOR3 *value) const
{
	ParameterRef *ref = m_parameters.get(parameter);

	if(ref)
	{
		ref->m_effect->SetVector(ref->m_handle, reinterpret_cast<const D3DXVECTOR4 *>(value));
	}
}

void ParametersPool::setValue(const std::string &parameter, const D3DXVECTOR4 *value) const
{
	ParameterRef *ref = m_parameters.get(parameter);

	if(ref)
	{
		ref->m_effect->SetVector(ref->m_handle, value);
	}
}

void ParametersPool::setValue(const std::string &parameter, const D3DXMATRIX *value) const
{
	ParameterRef *ref = m_parameters.get(parameter);

	if(ref)
	{
		ref->m_effect->SetMatrix(ref->m_handle, value);
	}
}