#include "Position.h"
#include "Size.h"
#include "Debug.h"
#include "MemoryLog.h"

Position::Position(const Position *parentPos, const Size &size, const D3DXVECTOR2 &offset, EAnchor relativePoint, EAnchor point):
m_parentPos(parentPos),
m_size(size),
m_offset(offset),
m_relativePoint(relativePoint),
m_point(point)
{
}

void Position::setPosition(const D3DXVECTOR2 &offset)
{
	m_offset = offset;
}

void Position::setPosition(const D3DXVECTOR2 &offset, EAnchor relativePoint, EAnchor point)
{
	m_offset = offset;
	m_relativePoint = relativePoint;
	m_point = point;
}

void Position::move(const D3DXVECTOR2 &distance)
{
	m_offset += distance;
}

D3DXVECTOR2 Position::getAnchorOffset(EAnchor anchor) const
{
	switch(anchor)
	{
	default:
	case GA_TOPLEFT:
		return D3DXVECTOR2(0.0f, 0.0f);
	case GA_TOP:
		return D3DXVECTOR2(m_size.getAbsoluteSize().x * 0.5f, 0.0f);
	case GA_TOPRIGHT:
		return D3DXVECTOR2(m_size.getAbsoluteSize().x, 0.0f);
	case GA_LEFT:
		return D3DXVECTOR2(0.0f, m_size.getAbsoluteSize().y  * 0.5f);
	case GA_CENTER:
		return m_size.getAbsoluteSize() * 0.5f;
	case GA_RIGHT:
		{
			const D3DXVECTOR2 &size = m_size.getAbsoluteSize();
			return D3DXVECTOR2(size.x, size.y * 0.5f);
		}
	case GA_BOTTOMLEFT:
		return D3DXVECTOR2(0.0f, m_size.getAbsoluteSize().y);
	case GA_BOTTOM:
		{
			const D3DXVECTOR2 &size = m_size.getAbsoluteSize();
			return D3DXVECTOR2(size.x * 0.5f, size.y);
		}
	case GA_BOTTOMRIGHT:
		return m_size.getAbsoluteSize();
	}
}

D3DXVECTOR2 Position::getAbsolutePosition() const
{
	if(m_relativePoint == GA_NONE)
	{
		return m_offset - getAnchorOffset(m_point);
	}
	else
	{
		Assert(m_parentPos != NULL);

		return m_parentPos->getAbsolutePosition() + m_parentPos->getAnchorOffset(m_relativePoint)
			+ m_offset - getAnchorOffset(m_point);
	}
}

void Position::setParentPosition(const Position *parentPos)
{
	m_parentPos = parentPos;
}