#ifndef H_RENDERER
#define H_RENDERER

#include <d3dx9.h>
#include <string>
#include "Buffer.h"
#include "Declaration.h"
#include "Shader.h"
#include "Texture.h"
#include "Image.h"
#include "ParametersPool.h"
#include "Font.h"

#define MAX_MATRIX_STACK_DEPTH	32

class Renderer
{
public:
	Renderer(LPDIRECT3DDEVICE9 device);
	~Renderer();

	const D3DXMATRIX *getMatrix() const;
	void setMatrix(const D3DXMATRIX &matrix);
	void multMatrix(const D3DXMATRIX &matrix);
	void popMatrix();
	void pushMatrix();

	template <class T>
	typename VertexBuffer<T>::Type * createVertexBuffer(unsigned size, const T *data = NULL);
	template <class T>
	typename IndexBuffer<T>::Type * createIndexBuffer(unsigned size, const T *data = NULL);

	template <int N>
	Declaration * createDeclaration(DeclarationElement (&decl)[N]);

	template <class T>
	void setVertexBuffer(unsigned stream, const typename VertexBuffer<T>::Type *buffer, unsigned start);
	void setDeclaration(const Declaration *declaration);

	void drawPrimitives(D3DPRIMITIVETYPE type, unsigned start, unsigned count);
	template <class T>
	void drawIndexedPrimitives(D3DPRIMITIVETYPE type, typename IndexBuffer<T>::Type *buffer, unsigned start, unsigned count);

	LPD3DXEFFECT createShader(const std::string &src);
	DynamicTexture * createDynamicTexture(const Image<unsigned short> &image);
	Texture * createTexture(const std::string &fileName);

	void setShader(Shader *shader);
	void setTexture(const std::string &name, const Texture *texture);

	const ParametersPool & getPool() const;

	Font * createFont(const std::string &fontName, unsigned height);
	void drawText(const std::string &text, const D3DXVECTOR2 &position, const Font &font, const D3DXVECTOR4 &color = D3DXVECTOR4(1.0f, 1.0f, 1.0f, 1.0f));

	void clearZBuffer();

private:
	LPDIRECT3DDEVICE9	 m_device;
	D3DXMATRIX			 m_matrixStack[MAX_MATRIX_STACK_DEPTH];
	unsigned			 m_matrixStackIndex;
	unsigned			 m_nbVertices;
	Shader				*m_shader;
	ParametersPool		 m_pool;
};

#include "Renderer.inl"

#endif