#include "MemoryLog.h"

template <class T>
typename VertexBuffer<T>::Type * Renderer::createVertexBuffer(unsigned size, const T *data)
{
	LPDIRECT3DVERTEXBUFFER9 d3dBuffer;
	m_device->CreateVertexBuffer(size * sizeof(T), D3DCREATE_HARDWARE_VERTEXPROCESSING | D3DUSAGE_WRITEONLY, 0,
		D3DPOOL_MANAGED, &d3dBuffer, NULL);
	VertexBuffer<T>::Type *buffer = new VertexBuffer<T>::Type(size, d3dBuffer);

	if(data)
	{
		buffer->fill(data, size);
	}
	return buffer;
}

template <class T>
struct IndexBufferFormat
{
	enum {
		Value = D3DFMT_INDEX16
	};
};

template <>
struct IndexBufferFormat<unsigned>
{
	enum {
		Value = D3DFMT_INDEX32
	};
};

template <class T>
typename IndexBuffer<T>::Type * Renderer::createIndexBuffer(unsigned size, const T *data)
{
	LPDIRECT3DINDEXBUFFER9 d3dBuffer;
	m_device->CreateIndexBuffer(size * sizeof(T), D3DCREATE_HARDWARE_VERTEXPROCESSING, static_cast<D3DFORMAT>(IndexBufferFormat<T>::Value),
		D3DPOOL_MANAGED, &d3dBuffer, NULL);
	IndexBuffer<T>::Type *buffer = new IndexBuffer<T>::Type(size, d3dBuffer);

	if(data)
	{
		buffer->fill(data, size);
	}
	return buffer;
}

template <int N>
Declaration * Renderer::createDeclaration(DeclarationElement (&decl)[N])
{
	return new Declaration(m_device, decl, N);
}

template <class T>
void Renderer::setVertexBuffer(unsigned stream, const typename VertexBuffer<T>::Type *buffer, unsigned start)
{
	m_device->SetStreamSource(stream, buffer->getBuffer(), start * sizeof(T), sizeof(T));
	m_nbVertices = buffer->getSize() - start;
}

template <class T>
void Renderer::drawIndexedPrimitives(D3DPRIMITIVETYPE type, typename IndexBuffer<T>::Type *buffer, unsigned start, unsigned count)
{
	m_device->SetIndices(buffer->getBuffer());
	m_shader->begin();
	do
	{
		m_device->DrawIndexedPrimitive(type, 0, 0, m_nbVertices, start, count);
	}
	while(m_shader->nextPass());
	m_shader->end();
}

#undef new