#ifndef H_RESOURCEMANAGER
#define H_RESOURCEMANAGER

#include "HashMap.h"
#include "Loader.h"
#include <vector>

template <class T>
class ResourceManager
{
public:
	ResourceManager(const std::string &basePath = "");
	~ResourceManager();

	T * getObject(const std::string &objectName, MediaManager &media);
	void removeReference(const T *ref);

	void registerLoader(const ILoader<T> *loader, const std::string &extensions);

	void onLostDevice();
	void onResetDevice();

private:
	ResourceManager(const ResourceManager<T> &);
	ResourceManager<T> & operator =(const ResourceManager<T> &);

	T * loadObject(const std::string &objectName, MediaManager &media);

	struct ResourceInfo
	{
		std::string	m_filename;
		unsigned	m_nbRefs;
	};

	HashMap<std::string, T *>						m_resources;
	HashMap<const T *, ResourceInfo>				m_infos;
	HashMap<std::string, const ILoader<T> *, 16>	m_loaders;
	std::vector<const ILoader<T> *>					m_loadersCopy;
	const std::string								m_basePath;
};

#include "ResourceManager.inl"

#endif