#include <algorithm>
#include <cctype>
#include "MemoryLog.h"

template <class T>
ResourceManager<T>::ResourceManager(const std::string &basePath):
m_basePath(basePath)
{
}

template <class T>
void deleteResource(T *&res)
{
	delete res;
}

template <class T>
ResourceManager<T>::~ResourceManager()
{
	m_resources.forEach(deleteResource<T>);
	for(std::vector<const ILoader<T> *>::iterator it = m_loadersCopy.begin(); it != m_loadersCopy.end(); ++it)
	{
		delete *it;
	}
}

template <class T>
T * ResourceManager<T>::getObject(const std::string &objectName, MediaManager &media)
{
	if(m_resources.isExist(objectName))
	{
		T * obj = m_resources[objectName];
		++m_infos[obj].m_nbRefs;
		return obj;
	}
	else
	{
		return loadObject(objectName, media);
	}
}

template <class T>
void ResourceManager<T>::removeReference(const T *ref)
{
	ResourceInfo &info = m_infos[ref];

	--info.m_nbRefs;
	if(!info.m_nbRefs)
	{
		m_resources.remove(info.m_filename);
		m_infos.remove(ref);
		delete ref;
	}
}

template <class T>
void ResourceManager<T>::registerLoader(const ILoader<T> *loader, const std::string &extensions)
{
	size_t end = 0;
	for(size_t start = 0; end != std::string::npos; start = end + 1)
	{
		end = extensions.find_first_of(';', start);
		std::string ext = extensions.substr(start, (end == std::string::npos) ? std::string::npos : (end - start));
		std::transform(ext.begin(), ext.end(), ext.begin(), std::tolower);
		m_loaders[ext] = loader;
	}
	m_loadersCopy.push_back(loader);
}

template <class T>
static void onLostDevice(T *&obj)
{
	obj->onLostDevice();
}

template <class T>
void ResourceManager<T>::onLostDevice()
{
	m_resources.forEach(::onLostDevice);
}

template <class T>
static void onResetDevice(T *&obj)
{
	obj->onResetDevice();
}

template <class T>
void ResourceManager<T>::onResetDevice()
{
	m_resources.forEach(::onResetDevice);
}

template <class T>
T * ResourceManager<T>::loadObject(const std::string &objectName, MediaManager &media)
{
	std::string ext = objectName.substr(objectName.find_last_of('.'));
	std::transform(ext.begin(), ext.end(), ext.begin(), std::tolower);

	T *object = m_loaders[ext]->load(m_basePath + objectName, media);
	ResourceInfo &info = m_infos[object];

	info.m_filename = objectName;
	info.m_nbRefs = 1;
	m_resources[objectName] = object;

	return object;
}

#undef new