#include "Road.h"
#include <exception>
#include "Debug.h"
#include "MemoryLog.h"

Road::Road(Crossing& c1, Crossing& c2, ERoadType type):
m_crossing1(&c1),
m_crossing2(&c2),
m_type(type),
m_zoneType(RZT_UNDEFINED),
m_isBridge(false)
{
}

Road::~Road(void)
{
}

/**************************************************************************************************
					Getters et Setters
**************************************************************************************************/

float Road::getWidth() const
{
	return (m_type == RT_STREET) ? 0.4f : 0.9f;
}

float Road::get2DLength() const
{
	const D3DXVECTOR2 &vec = get2DVector();

	return D3DXVec2Length(&vec);
}

D3DXVECTOR2 Road::get2DVector() const
{
	return m_crossing2->get2DPosition() - m_crossing1->get2DPosition();
}

D3DXVECTOR3 Road::get3DVector(const Crossing *start) const
{
	return getEndCrossing(*start).getPosition() - start->getPosition();
}

ERoadType Road::getType() const
{
	return m_type;
}

const Crossing& Road::getEndCrossing(const Crossing &startCrossing) const
{
	// on retourne le croisement sur lequel on n'est pas
	if (&startCrossing == m_crossing1)
	{
		return *m_crossing2;
	}
	if (&startCrossing == m_crossing2)
	{
		return *m_crossing1;
	}
	throw new std::exception("Crossing not on road");
}

const Crossing& Road::getEndCrossing() const
{
	return *m_crossing2;
}

const Crossing& Road::getStartCrossing() const
{
	return *m_crossing1;
}

Crossing & Road::getEndCrossing()
{
	return *m_crossing2;
}

Crossing & Road::getStartCrossing()
{
	return *m_crossing1;
}

void Road::setCrossing(Crossing& oldCrossing, Crossing& newCrossing)
{
	if (&oldCrossing == m_crossing1)
	{
		m_crossing1 = &newCrossing;
	}
	else if (&oldCrossing == m_crossing2)
	{
		m_crossing2 = &newCrossing;
	}
	else
	{
		throw new std::exception("Crossing not on road");
	}
}

void Road::detach() const
{
	m_crossing1->removeRoad(*this);
	m_crossing2->removeRoad(*this);
}

bool Road::isBridge() const
{
	return m_isBridge;
}

void Road::setBridge()
{
	m_isBridge = true;
}

void Road::addZoneType(ERoadZoneType type)
{
	m_zoneType |= type;
}

unsigned Road::getZoneType() const
{
	return m_zoneType;
}