#ifndef H_ROAD
#define H_ROAD

#include "Crossing.h"
#include <vector>

class Agent;

enum ERoadType
{
	RT_STREET,
	RT_HIGHWAY
};

enum ERoadZoneType
{
	RZT_UNDEFINED	= 0x01,
	RZT_COMMERCIAL	= 0x02,
	RZT_RESIDENTIAL	= 0X04
};

class Road
{
public:
	Road(Crossing& c1, Crossing& c2, ERoadType type);
	~Road(void);
	/* getters et setters */
	float getWidth() const;
	float get2DLength() const;
	D3DXVECTOR2 get2DVector() const;
	D3DXVECTOR3 get3DVector(const Crossing *start) const;
	ERoadType getType() const;
	/* getEndCrossing
	Retourne le croisement de fin de la route. startCrossing correspond au croisement de dpart. */
	const Crossing& getEndCrossing(const Crossing& startCrossing) const;
	/* pour le moment, on considre par dfaut que le croisement de dpart est le 1 et celui d'arrive le 2.
	Si il y a mise en place d'un sens de circulation,  modifier */
	const Crossing& getEndCrossing() const;
	const Crossing& getStartCrossing() const;
	Crossing & getEndCrossing();
	Crossing & getStartCrossing();
	/* setCrossing
	Remplace le croisement oldCrossing par newCrossing */
	void setCrossing(Crossing& oldCrossing, Crossing& newCrossing);
	void detach() const;

	bool isBridge() const;
	void setBridge();
	void addZoneType(ERoadZoneType type);
	unsigned getZoneType() const;

private:
	Crossing		*m_crossing1,
					*m_crossing2;
	ERoadType		 m_type;
	unsigned		 m_zoneType;
	bool			 m_isBridge;
};

#endif