#ifndef H_ROADNETWORK
#define H_ROADNETWORK

#include <vector>
#include "Buffer.h"
#include <d3dx9.h>

class Road;
class MediaManager;
class Renderer;
class Declaration;
class Shader;
class Crossing;

class RoadNetwork
{
public:
	RoadNetwork(MediaManager &media);
	~RoadNetwork();

	void addRoad(Road &road);
	void addCrossing(Crossing &c);
	Road * divideRoad(Road &road, Crossing &c);

	void sortRoadsOnCrossing();
	void removeOneway();
	void commitChange();
	void render(Renderer &renderer) const;

	void reset();

	const std::vector<Road*> & getRoads() const;
	const std::vector<Crossing*> & getCrossings() const;
	std::vector<Crossing *> & getCrossings();

protected:
	void recomputeBuffer();

private:
	struct Vertex
	{
		D3DXVECTOR3 position;
		D3DXVECTOR3 color;
	};

	VertexBuffer<Vertex>::Type			*m_vb;
	IndexBuffer<unsigned short>::Type	*m_ib;
	Shader								*m_shader;
	Declaration							*m_declaration;
	MediaManager						*m_media;
	std::vector<Road *>					 m_roads;
	std::vector<Crossing *>				 m_crossings;
};

#endif