#include "Shader.h"
#include "Texture.h"
#include "MemoryLog.h"

Shader::Shader(LPD3DXEFFECT effect):
m_effect(effect)
{
	getHandles();
}

Shader::~Shader()
{
	m_effect->Release();
}

void Shader::setValue(D3DXHANDLE parameterBlock) const
{
	m_effect->ApplyParameterBlock(parameterBlock);
}

void Shader::setValue(const std::string &parameter, float value) const
{
	D3DXHANDLE *handle = m_handles.get(parameter);

	if(handle)
	{
		m_effect->SetFloat(*handle, value);
	}
}

void Shader::setValue(const std::string &parameter, const D3DXVECTOR2 *value) const
{
	D3DXHANDLE *handle = m_handles.get(parameter);

	if(handle)
	{
		m_effect->SetVector(*handle, reinterpret_cast<const D3DXVECTOR4 *>(value));
	}
}

void Shader::setValue(const std::string &parameter, const D3DXVECTOR3 *value) const
{
	D3DXHANDLE *handle = m_handles.get(parameter);

	if(handle)
	{
		m_effect->SetVector(*handle, reinterpret_cast<const D3DXVECTOR4 *>(value));
	}
}

void Shader::setValue(const std::string &parameter, const D3DXVECTOR4 *value) const
{
	D3DXHANDLE *handle = m_handles.get(parameter);

	if(handle)
	{
		m_effect->SetVector(*handle, value);
	}
}

void Shader::setValue(const std::string &parameter, const D3DXMATRIX *value) const
{
	D3DXHANDLE *handle = m_handles.get(parameter);

	if(handle)
	{
		m_effect->SetMatrix(*handle, value);
	}
}

void Shader::setValue(const std::string &parameter, const D3DXVECTOR4 *value, unsigned count) const
{
	D3DXHANDLE *handle = m_handles.get(parameter);

	if(handle)
	{
		m_effect->SetVectorArray(*handle, value, count);
	}
}

void Shader::setTexture(const std::string &parameter, const Texture *texture) const
{
	D3DXHANDLE *handle = m_handles.get(parameter);

	if(handle)
	{
		m_effect->SetTexture(*handle, texture->getData());
	}
}

void Shader::begin()
{
	m_effect->Begin(&m_nbPasses, D3DXFX_DONOTSAVESTATE);
	m_effect->BeginPass(0);
	m_actPass = 0;
}

void Shader::end()
{
	m_effect->End();
}

bool Shader::nextPass()
{
	m_effect->EndPass();
	if(++m_actPass < m_nbPasses)
	{
		m_effect->BeginPass(m_actPass);
		return true;
	}
	return false;
}

void Shader::onLostDevice()
{
	m_effect->OnLostDevice();
}

void Shader::onResetDevice()
{
	m_effect->OnResetDevice();
}

void Shader::getHandles()
{
	D3DXHANDLE parameterHandle;
	D3DXPARAMETER_DESC desc;
	D3DXEFFECT_DESC effectDesc;

	m_effect->GetDesc(&effectDesc);
	for(unsigned i = 0; i < effectDesc.Parameters; ++i)
	{
		parameterHandle = m_effect->GetParameter(NULL, i);
		m_effect->GetParameterDesc(parameterHandle, &desc);

		m_handles[desc.Name] = parameterHandle;
	}
}

