#include "Size.h"
#include "GUIWindow.h"
#include "Debug.h"
#include "MemoryLog.h"

Size::Size(const Size *parentSize):
m_parentSize(parentSize),
m_offset(0.0f, 0.0f),
m_scale(0.0f, 0.0f)
{
}

Size::Size(const Size *parentSize, const D3DXVECTOR2 &absoluteSize):
m_parentSize(parentSize),
m_offset(absoluteSize),
m_scale(0.0f, 0.0f)
{
}

Size::Size(const Size *parentSize, const D3DXVECTOR2 &offset, const D3DXVECTOR2 &scale):
m_parentSize(parentSize),
m_offset(offset),
m_scale(scale)
{
}

void Size::setAbsoluteSize(const D3DXVECTOR2 &size)
{
	m_offset = size;
	m_scale = D3DXVECTOR2(0.0f, 0.0f);
}

void Size::setRelativeSize(const D3DXVECTOR2 &offset, const D3DXVECTOR2 &scale)
{
	m_offset = offset;
	m_scale = scale;
}

void Size::setAbsoluteSizeX(float size)
{
	m_offset.x = size;
	m_scale.x = 0.0f;
}

void Size::setAbsoluteSizeY(float size)
{
	m_offset.y = size;
	m_scale.y = 0.0f;
}

void Size::setRelativeSizeX(float offset, float scale)
{
	m_offset.x = offset;
	m_scale.x = scale;
}

void Size::setRelativeSizeY(float offset, float scale)
{
	m_offset.y = offset;
	m_scale.y = scale;
}

D3DXVECTOR2 Size::getAbsoluteSize() const
{
	if(m_scale == D3DXVECTOR2(0.0f, 0.0f))
	{
		return m_offset;
	}
	else
	{
		Assert(m_parentSize != NULL);

		D3DXVECTOR2 size(m_parentSize->getAbsoluteSize());
		size.x = size.x * m_scale.x + m_offset.x;
		size.y = size.y * m_scale.y + m_offset.y;

		return size;
	}
}

void Size::setParentSize(const Size *parentSize)
{
	m_parentSize = parentSize;
}