#include "Sound.h"
#include <fstream>
#include <xaudio2.h>
#include "Debug.h"
#include "MemoryLog.h"

#define CHUNK_ID(char0, char1, char2, char3) (char0 | char1 << 8 | char2 << 16 | char3 << 24)

struct ChunkHeader
{
	unsigned	id;
	unsigned	size;
	unsigned	offset;
};

struct RiffFileHeader
{
	unsigned id;
	unsigned size;
	unsigned format;
};

bool readChunkHeader(std::ifstream &file, ChunkHeader &header)
{
	if(!file.read(reinterpret_cast<char *>(&header), 8))
	{
		return false;
	}
	header.offset = file.tellg();
	return true;
}

void readChunk(std::ifstream &file, const ChunkHeader &header, char *buffer)
{
	file.seekg(header.offset);
	file.read(buffer, header.size);
}

void findChunk(std::ifstream &file, ChunkHeader &target)
{
	file.seekg(12);
	ChunkHeader header;

	while(readChunkHeader(file, header))
	{
		if(target.id == header.id)
		{
			target = header;
			return;
		}
		file.seekg(header.offset + header.size);
	}
}

bool isValidWave(std::ifstream &file)
{
	RiffFileHeader header;

	file.seekg(0);
	file.read(reinterpret_cast<char *>(&header), sizeof(RiffFileHeader));

	return (header.id == CHUNK_ID('R', 'I', 'F', 'F') && header.format == CHUNK_ID('W', 'A', 'V', 'E'))	;
}

Sound::Sound(const std::string &filename):
m_bufferData(NULL)
{
	std::ifstream file(filename.c_str(), std::ios_base::binary | std::ios_base::in);

	if(!file.is_open())
	{
		throw std::exception(("File " + filename + " not found").c_str());
	}
	
	if(!isValidWave(file))
	{
		throw std::exception(("File " + filename + " is invalid wave file").c_str());
	}

	ChunkHeader header = { CHUNK_ID('f', 'm', 't', ' '), 0, 0 };
	findChunk(file, header);
	readChunk(file, header, reinterpret_cast<char *>(&m_format));
	m_format.cbSize = 0;

	header.id = CHUNK_ID('d', 'a', 't', 'a');
	findChunk(file, header);
	m_dataLength = header.size;
	m_bufferData = new char[m_dataLength];
	readChunk(file, header, m_bufferData);
}

Sound::~Sound()
{
	delete[] m_bufferData;
}

const WAVEFORMATEX & Sound::getFormat() const
{
	return m_format;
}

unsigned Sound::getDataLength() const
{
	return m_dataLength;
}

const char * Sound::getData() const
{
	return m_bufferData;
}