#ifndef H_SPEAKER
#define H_SPEAKER

#include <d3dx9math.h>

struct IXAudio2;
struct IXAudio2SourceVoice;
struct IXAudio2SubmixVoice;
class Sound;
class SoundEngine;

enum ESpeakerState
{
	STOP,
	PLAYING,
	DESACTIVATE
};

class Speaker
{
public:
	Speaker(const Sound *sound, IXAudio2 *device, SoundEngine *engine);
	~Speaker();

	void play(bool looped);
	void stop();

	void activate();
	void desactivate();
	void computeRealVolume(const D3DXVECTOR3 &position);
	void update(const D3DXVECTOR3 &position, const D3DXMATRIX &viewMatrix, const D3DXVECTOR3 &cameraVelocity, float elapsedTime);

	void attachPosition(const D3DXVECTOR3 &position);
	void setVolume(float volume);
	const D3DXVECTOR3 & getPosition() const;
	float getRealVolume() const;
	bool isPlaying() const;

private:
	const Sound			*m_sound;
	IXAudio2SourceVoice	*m_voice;
	SoundEngine			*m_engine;
	const D3DXVECTOR3	*m_position;
	D3DXVECTOR3			 m_lastPosition;
	float				 m_volume;
	float				 m_realVolume;
	ESpeakerState		 m_state;
	bool				 m_looped;
};

#endif