#ifndef H_Way
#define H_Way

#include <vector>
#include "Road.h"
#include <d3dx9math.h>

class Junction;
class Agent;

class Way
{
public:
	Way(Junction* const begin, Junction* const end, ERoadType type, unsigned zone);
	~Way(void);

	void addAgent (const Agent& agent);
	void removeAgent (const Agent& agent);
	const Agent* agentAhead(const Agent& agent) const;
	const Agent* agentAhead(float positionOnRoad) const;
	const Agent* agentAhead() const;
	const Agent* agentBehind() const;
	float timeBeforeNextAgent() const;
	bool isCrossed(const Way& way) const;

	const std::vector<const Agent*>& getAgents() const;
	const Junction& getBegin() const;
	Junction& getBegin();
	const Junction& getEnd() const;
	Junction& getEnd();
	float getLength() const;
	const D3DXVECTOR3& getVector() const;
	float getSpeedLimit() const;
	ERoadType getRoadType() const;

	bool isZoneType(ERoadZoneType type) const;

private:
	Junction					*m_begin;
	Junction					*m_end;
	std::vector<const Agent*>	 m_agents;
	D3DXVECTOR3					 m_vector;
	float						 m_length;
	ERoadType					 m_type;
	unsigned					 m_zone;
	
	float pTimeBeforeNextAgent(unsigned execNb) const;
};

#endif